var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>DAG paths</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>DAG paths</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"dag-paths\"></a></div>\n\
<p>A path through the DAG is a set of nodes which uniquely identifies the location of a particular node or instance of a node in the graph. The path represents a graph ancestry beginning with the root node of the graph and containing, in succession, a particular child of the root node followed by a particular child of this child, etc., down to the node identified by the path. For instanced nodes, there are multiple paths which lead from the root node to the instanced node, one path for each instance. Paths are displayed in Maya by naming each node in the path starting with the root node and separated by the vertical line character, &quot;|&quot;.</p>\n\
<div class=\'section\'><a id=\"dag-paths-and-worldspace-operations-in-the-api\"></a><h2 id=\"dag-paths-and-worldspace-operations-in-the-api\">DAG paths and worldspace operations in the API</h2></div>\n\
<p>It is important to note that because the DAG path represents how a shape is inserted into the scene, a DAG path must be used when attempting any world space operation via the API. If one simply gets an &quot;<span class=\'code\'>MObject</span>&quot; handle to a node and asks for the world space position of a component of that node, the API operation will fail. This is because without the DAG path Maya has no idea where in world space the object is. In fact, in the case of instanced objects, there can be multiple answers to that question, and only the DAG path will uniquely identify the particular instance of the node. Almost all of the classes that contains methods that will return an <span class=\'code\'>MObject</span> for a node also contain methods that will return DAG paths so you can get an <span class=\'code\'>MDagPath</span> handle to the desired node. As well, all the <span class=\'code\'>MFn</span> classes can be constructed with either an <span class=\'code\'>MObject</span> or an <span class=\'code\'>MDagPath</span>. If an <span class=\'code\'>MObject</span> is used, all world space operations attempted using methods of the <span class=\'code\'>MFn</span> class will fail, if an <span class=\'code\'>MDagPath</span> is used, they will succeed.</p>\n\
<div class=\'section\'><a id=\"adding-or-removing-nodes-from-the-representation\"></a><h2 id=\"adding-or-removing-nodes-from-the-representation\">Adding or removing nodes from the representation</h2></div>\n\
<p>The <span class=\'code\'>MDagPath</span> class builds a representation of a path and lets you examine and modify this representation. The <span class=\'code\'>MDagPath</span> represents paths as a stack of nodes with the root node being on the bottom of the stack. The <span class=\'code\'>push()</span> and <span class=\'code\'>pop()</span> methods allow nodes along the path to be added or removed from the representation.</p>\n\
<blockquote>\n\
<p><strong>Note:</strong> These methods do not add and remove nodes from the actual DAG but only from the representation constructed by the <span class=\'code\'>MDagPath</span> class. The comparison and assignment operators operate on the representations constructed by the <span class=\'code\'>MDagPath</span> class and not on the graph itself. So assigning one path to another will not modify the graph but only the contents of the destination <span class=\'code\'>MDagPath</span> instance.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"inclusive-and-exclusive-matrices\"></a><h2 id=\"inclusive-and-exclusive-matrices\">Inclusive and exclusive matrices</h2></div>\n\
<p>Since the nodes in a path exist at different levels of the DAG hierarchy, there is a different transformation that may have accumulated at each node in the path. The <span class=\'code\'>MDagPath</span> class allows these transformations to be returned using the <span class=\'code\'>inclusiveMatrix()</span> and <span class=\'code\'>exclusiveMatrix()</span> classes.</p>\n\
<ul>\n\
<li>An &quot;inclusive matrix&quot; represents the accumulated transformation to the last node stored in the DAG path taking into account the last node.</li>\n\
<li>An &quot;exclusive matrix&quot; represents the same accumulation with the exception that it does not take into account any transformation from the last node.</li>\n\
</ul>\n\
<p>For example, if a path is defined as:</p>\n\
<p>|RootTransform|Transform1|Transform2|Shape</p>\n\
<p>the inclusive matrix down to Transform2 would be the accumulation of RootTransform, Transform1, and Transform2. The exclusive matrix would contain the accumulation of only RootTransform and Transform1.</p>\n\
<div class=\'section\'><a id=\"why-add-the-shape-node-to-a-dag-path\"></a><h2 id=\"why-add-the-shape-node-to-a-dag-path\">Why add the shape node to a DAG path</h2></div>\n\
<p>In Maya, selection at the object level results in the selection of the transform node that is the parent node of the shape actually selected. When querying the selection using <span class=\'code\'>MGlobal::getActiveSelectionList()</span>, the <span class=\'code\'>MDagPath</span> returned to the caller only specifies the path to this transform and not down to the actual shape that was picked on the screen. A convenience method on <span class=\'code\'>MDagPath</span> called <span class=\'code\'>extendToShape()</span> can be called to add the shape node below the last transform to the path.</p>\n\
<p>The valid function sets applicable to a particular <span class=\'code\'>MDagPath</span> are determined by the last node on the path. If the last node is a transform node, then the function sets that can operate on transform nodes can be applied to the <span class=\'code\'>MDagPath</span> instance. If a shape node is the last node of the path, then the applicable function sets for the <span class=\'code\'>MDagPath</span> instance are those sets which can operate on the shape.</p>\n\
<div class=\'section\'><a id=\"unique-names\"></a><h2 id=\"unique-names\">Unique Names</h2></div>\n\
<p>The use of the DAG path allows for object names to be reused. Object names can be reused as long as the same name does not appear on more than one DAG node with a common parent.</p>\n\
<div class=\'figure\'><img src=\'dev_help/images/comp_Transform02.png\' title=\'\'></div>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";