var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>DAG walking example</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>DAG walking example</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"dag-walking-example\"></a></div>\n\
<p>The following example is the scanDagSyntaxCmd example. It demonstrates iterating through the DAG in either a depth first, or a breadth first manner. This code makes a good basis for many DAG walking plug-ins, in particular those written as file translators.</p>\n\
<p>As with previous examples, the list of include files is omitted for brevity. See the <span class=\'code\'>scanDagSyntaxCmd.cpp</span> file in the <span class=\'code\'>devkit/plug-ins</span> folder of your Developer Kit installation (see <a href=\'#!/url=./dev_help/Setting-up-your-build/Windows-environment-64-bit-.html\' title=\'\'>Setting up your build environment: Windows environment (64-bit)</a>) for the complete example.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">class scanDagSyntax: public MPxCommand\n\
{\n\
 public:\n\
     scanDagSyntax() {};\n\
     virtual ~scanDagSyntax();\n\
     static void* creator();\n\
     static MSyntax newSyntax();\n\
     virtual MStatus doIt( const MArgList&amp; );\n\
     //This is a simple example so the undoIt() and redoIt() methods are not implemented. \n\
 private:\n\
     MStatus parseArgs( const MArgList&amp; args,\n\
     MItDag::TraversalType&amp; traversalType,\n\
     MFn::Type&amp; filter, bool &amp; quiet);\n\
     MStatus doScan( const MItDag::TraversalType traversalType,\n\
     MFn::Type filter, bool quiet);\n\
     void printTransformData(const MDagPath&amp; dagPath, bool quiet);\n\
};\n\
\n\
scanDagSyntax::~scanDagSyntax() {}\n\
void* scanDagSyntax::creator()\n\
{\n\
    return new scanDagSyntax;\n\
}\n\
\n\
MSyntax scanDagSyntax::newSyntax()\n\
{\n\
    MSyntax syntax;\n\
    syntax.addFlag(kBreadthFlag, kBreadthFlagLong);\n\
    syntax.addFlag(kDepthFlag, kDepthFlagLong);\n\
    syntax.addFlag(kCameraFlag, kCameraFlagLong);\n\
    syntax.addFlag(kLightFlag, kLightFlagLong);\n\
    syntax.addFlag(kNurbsSurfaceFlag, kNurbsSurfaceFlagLong);\n\
    syntax.addFlag(kQuietFlag, kQuietFlagLong);\n\
    return syntax;\n\
}\n\
\n\
MStatus scanDagSyntax::doIt( const MArgList&amp; args )\n\
{\n\
     MItDag::TraversalType traversalType = MItDag::kDepthFirst;\n\
     MFn::Type filter = MFn::kInvalid;\n\
     MStatus status;\n\
     bool quiet = false;\n\
\n\
</pre></div><p>The DAG iterator being used later can be set to only iterate across objects of a particular type (for example cameras). If the filter mode is set to <span class=\'code\'>MFn::kInvalid</span>, no filtering will be done and all DAG nodes will be iterated across.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">     status = parseArgs ( args, traversalType, filter, quiet );\n\
     if (!status)\n\
         return status;\n\
     return doScan( traversalType, filter, quiet);\n\
};\n\
//The doIt() method is simply calling a few auxiliary methods which do the real work.\n\
MStatus scanDagSyntax::parseArgs( const MArgList&amp; args,\n\
                                  MItDag::TraversalType&amp; traversalType,\n\
                                  MFn::Type&amp; filter,\n\
                                  bool &amp; quiet)\n\
{\n\
    MStatus stat;\n\
    MArgDatabase argData(syntax(), args);\n\
    MString arg;\n\
    if (argData.isFlagSet(kBreadthFlag))\n\
        traversalType = MItDag::kBreadthFirst;\n\
    else if (argData.isFlagSet(kDepthFlag))\n\
        traversalType = MItDag::kDepthFirst;\n\
    if (argData.isFlagSet(kCameraFlag))\n\
        filter = MFn::kCamera;\n\
    else if (argData.isFlagSet(kLightFlag))\n\
        filter = MFn::kLight;\n\
    else if (argData.isFlagSet(kNurbsSurfaceFlag))\n\
        filter = MFn::kNurbsSurface;\n\
    if (argData.isFlagSet(kQuietFlag))\n\
        quiet = true;\n\
    return stat;\n\
}\n\
\n\
</pre></div><p>The DAG iterator can either iterate across the DAG depth first or breadth first. This simple example only filters on cameras, lights, and NURBS surfaces, but it is possible to iterate across any type in <span class=\'code\'>MFn::Type</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus scanDagSyntax::doScan( const MItDag::TraversalType traversalType,\n\
                               MFn::Type filter,\n\
                               bool quiet)\n\
{ \n\
\n\
</pre></div><p>This method will do all the real work of this command. It uses the traversal type (depth or breadth first) and the filter type to initialize an MItDag (a DAG iterator) to walk across the DAG.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">    MStatus status;\n\
    MItDag dagIterator( traversalType, filter, &amp;status);\n\
\n\
</pre></div><p>The DAG iterator is initialized looking at the DAG. It will walk the DAG downwards.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">    if ( !status) {\n\
        status.perror(&quot;MItDag constructor&quot;);\n\
        return status;\n\
    }\n\
 // Scan the entire DAG and output the name and depth of each node\n\
\n\
    if (traversalType == MItDag::kBreadthFirst)\n\
        if (!quiet)\n\
            cout &lt;&lt; endl &lt;&lt; &quot;Starting Breadth First scan of the Dag&quot;;\n\
        else\n\
    if (!quiet)\n\
        cout &lt;&lt; endl &lt;&lt; &quot;Starting Depth First scan of the Dag&quot;;\n\
\n\
</pre></div><p>Breadth first walking of the DAG means that siblings will be visited before children, while depth first means that children will be visited before siblings.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">    switch (filter) {\n\
    case MFn::kCamera:\n\
        if (!quiet)\n\
            cout &lt;&lt; &quot;: Filtering for Cameras\\n&quot;;\n\
        break;\n\
    case MFn::kLight:\n\
        if (!quiet)\n\
            cout &lt;&lt; &quot;: Filtering for Lights\\n&quot;;\n\
        break;\n\
    case MFn::kNurbsSurface:\n\
        if (!quiet)\n\
            cout &lt;&lt; &quot;: Filtering for Nurbs Surfaces\\n&quot;;\n\
        break;\n\
    default:\n\
        cout &lt;&lt; endl;\n\
    }\n\
    int objectCount = 0;\n\
    for ( ; !dagIterator.isDone(); dagIterator.next() ) {\n\
        MDagPath dagPath;\n\
        status = dagIterator.getPath(dagPath);\n\
        if ( !status ) {\n\
            status.perror(&quot;MItDag::getPath&quot;);\n\
            continue;\n\
        }\n\
\n\
</pre></div><p><span class=\'code\'>MItDag::getPath()</span> gets the reference to the object that the iterator is currently on. This DAG path can then be used in a function set to operate on the object. In general it is not a good idea to rearrange the DAG from with an iterator.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">        MFnDagNode dagNode(dagPath, &amp;status);\n\
        if ( !status ) {\n\
            status.perror(&quot;MFnDagNode constructor&quot;);\n\
            continue;\n\
        }\n\
        if (!quiet)\n\
            cout &lt;&lt; dagNode.name() &lt;&lt; &quot;: &quot; &lt;&lt; dagNode.typeName() &lt;&lt; endl;\n\
        if (!quiet)\n\
            cout &lt;&lt; &quot; dagPath: &quot; &lt;&lt; dagPath.fullPathName() &lt;&lt; endl;\n\
        objectCount += 1;\n\
        if (dagPath.hasFn(MFn::kCamera)) {\n\
\n\
</pre></div><p>This determines if the object the iterator is currently visiting is a camera or not, and if it is, the following code outputs camera specific information.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">            MFnCamera camera (dagPath, &amp;status);\n\
            if ( !status ) {\n\
                status.perror(&quot;MFnCamera constructor&quot;);\n\
                continue;\n\
            }\n\
            // Get the translation/rotation/scale data\n\
            printTransformData(dagPath, quiet);\n\
            // Extract some interesting Camera data\n\
            if (!quiet)\n\
            {\n\
                cout &lt;&lt; &quot; eyePoint: &quot;\n\
                     &lt;&lt; camera.eyePoint(MSpace::kWorld) &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; upDirection: &quot;\n\
                     &lt;&lt; camera.upDirection(MSpace::kWorld) &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; viewDirection: &quot;\n\
                     &lt;&lt; camera.viewDirection(MSpace::kWorld) &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; aspectRatio: &quot; &lt;&lt; camera.aspectRatio() &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; horizontalFilmAperture: &quot;\n\
                     &lt;&lt; camera.horizontalFilmAperture() &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; verticalFilmAperture: &quot;\n\
                     &lt;&lt; camera.verticalFilmAperture() &lt;&lt; endl;\n\
            }\n\
        } \n\
        else if (dagPath.hasFn(MFn::kLight)) {\n\
\n\
</pre></div><p>If the object is a light, this code outputs light specific information.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">            MFnLight light (dagPath, &amp;status);\n\
            if ( !status ) {\n\
                    status.perror(&quot;MFnLight constructor&quot;);\n\
                    continue;\n\
            }\n\
            // Get the translation/rotation/scale data\n\
            printTransformData(dagPath, quiet);\n\
            // Extract some interesting Light data\n\
            MColor color;\n\
            color = light.color();\n\
            if (!quiet)\n\
            {\n\
                cout &lt;&lt; &quot; color: [&quot;\n\
                &lt;&lt; color.r &lt;&lt; &quot;, &quot;\n\
                &lt;&lt; color.g &lt;&lt; &quot;, &quot;\n\
                &lt;&lt; color.b &lt;&lt; &quot;]\\n&quot;;\n\
            }\n\
            color = light.shadowColor();\n\
            if (!quiet)\n\
            {\n\
                cout &lt;&lt; &quot; shadowColor: [&quot;\n\
                &lt;&lt; color.r &lt;&lt; &quot;, &quot;\n\
                &lt;&lt; color.g &lt;&lt; &quot;, &quot;\n\
                &lt;&lt; color.b &lt;&lt; &quot;]\\n&quot;;\n\
                cout &lt;&lt; &quot; intensity: &quot; &lt;&lt; light.intensity() &lt;&lt; endl;\n\
            }\n\
        } \n\
        else if (dagPath.hasFn(MFn::kNurbsSurface)) {\n\
            //Finally, if the object is a NURBS surface, surface specific information is output.\n\
            MFnNurbsSurface surface (dagPath, &amp;status);\n\
            if ( !status ) {\n\
                status.perror(&quot;MFnNurbsSurface constructor&quot;);\n\
                continue;\n\
            }\n\
            // Get the translation/rotation/scale data\n\
            printTransformData(dagPath, quiet);\n\
            // Extract some interesting Surface data\n\
            if (!quiet)\n\
            {\n\
                cout &lt;&lt; &quot; numCVs: &quot;\n\
                     &lt;&lt; surface.numCVsInU()\n\
                     &lt;&lt; &quot; * &quot;\n\
                     &lt;&lt; surface.numCVsInV()\n\
                     &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; numKnots: &quot;\n\
                     &lt;&lt; surface.numKnotsInU()\n\
                     &lt;&lt; &quot; * &quot;\n\
                     &lt;&lt; surface.numKnotsInV()\n\
                     &lt;&lt; endl;\n\
                cout &lt;&lt; &quot; numSpans: &quot;\n\
                     &lt;&lt; surface.numSpansInU()\n\
                     &lt;&lt; &quot; * &quot;\n\
                     &lt;&lt; surface.numSpansInV()\n\
                     &lt;&lt; endl;\n\
            }\n\
        } \n\
        else {\n\
\n\
</pre></div><p>For any other type of DAG node, just the transformation information is printed.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">            // Get the translation/rotation/scale data\n\
            printTransformData(dagPath, quiet);\n\
        }\n\
    }\n\
    if (!quiet)\n\
    {\n\
        cout.flush();\n\
    }\n\
    setResult(objectCount);\n\
    return MS::kSuccess;\n\
}\n\
\n\
void scanDagSyntax::printTransformData(const MDagPath&amp; dagPath, bool quiet)\n\
{\n\
    //This method simply determines the transformation information on the DAG node and prints it out.\n\
    MStatus status;\n\
    MObject transformNode = dagPath.transform(&amp;status);\n\
    // This node has no transform - i.e., it’s the world node\n\
    if (!status &amp;&amp; status.statusCode () == MStatus::kInvalidParameter)\n\
        return;\n\
    MFnDagNode transform (transformNode, &amp;status);\n\
    if (!status) {\n\
        status.perror(&quot;MFnDagNode constructor&quot;);\n\
        return;\n\
    }\n\
    MTransformationMatrix matrix (transform.transformationMatrix());\n\
    if (!quiet)\n\
    {\n\
        cout &lt;&lt; &quot; translation: &quot; &lt;&lt; matrix.translation(MSpace::kWorld)\n\
            &lt;&lt; endl;\n\
    }\n\
    double threeDoubles[3];\n\
    MTransformationMatrix::RotationOrder rOrder;\n\
    matrix.getRotation (threeDoubles, rOrder, MSpace::kWorld);\n\
    if (!quiet)\n\
    {\n\
        cout &lt;&lt; &quot; rotation: [&quot;\n\
             &lt;&lt; threeDoubles[0] &lt;&lt; &quot;, &quot;\n\
             &lt;&lt; threeDoubles[1] &lt;&lt; &quot;, &quot;\n\
             &lt;&lt; threeDoubles[2] &lt;&lt; &quot;]\\n&quot;;\n\
    }\n\
    matrix.getScale (threeDoubles, MSpace::kWorld);\n\
    if (!quiet)\n\
    {\n\
        cout &lt;&lt; &quot; scale: [&quot;\n\
             &lt;&lt; threeDoubles[0] &lt;&lt; &quot;, &quot;\n\
             &lt;&lt; threeDoubles[1] &lt;&lt; &quot;, &quot;\n\
             &lt;&lt; threeDoubles[2] &lt;&lt; &quot;]\\n&quot;;\n\
    }\n\
}\n\
\n\
MStatus initializePlugin( MObject obj )\n\
{ \n\
    MStatus status;\n\
    MFnPlugin plugin ( obj, &quot;Autodesk - Example&quot;, &quot;2.0&quot;, &quot;Any&quot; );\n\
    status = plugin.registerCommand( &quot;scanDagSyntax&quot;, \n\
    scanDagSyntax::creator,\n\
    scanDagSyntax::newSyntax ); \n\
    return status;\n\
}\n\
\n\
MStatus uninitializePlugin( MObject obj )\n\
{\n\
    MStatus status;\n\
    MFnPlugin plugin( obj );\n\
    status = plugin.deregisterCommand( &quot;scanDagSyntax&quot; );\n\
    return status;\n\
}\n\
\n\
</pre></div><p>The plug-in finishes with the usual <span class=\'code\'>initializePlugin</span> and <span class=\'code\'>uninitializePlugin</span> methods.</p>\n\
<p>This plug-in can easily be modified for use as a file translator, or any other type of plug-in which needs to visit the DAG nodes in the model.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";