var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Data blocks and data handles</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Data blocks and data handles</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"data-blocks-and-data-handles\"></a></div>\n\
<p>A node&#39;s attribute values are stored in its data block. The node owns its data block and uses it for computation. The data block is passed to <a href=\'#!/url=./dev_help/Dependency-graph-plug-ins/Implementing-the-compute-method.html\' title=\'\'>the node&#39;s <span class=\'code\'>compute()</span> method</a> when the node receives <a href=\'#!/url=./dev_help/Dependency-graph-plug-ins/DirtyPropagation.html\' title=\'\'>a request to update one of its output plug</a>. This will happen when the underlying framework updates any of the node&#39;s input values within its data block.</p>\n\
<p>A data handle is used to access the data in the <span class=\'code\'>MDataBlock</span> object. An <span class=\'code\'>MDataHandle</span> object is a reference to a specific piece of data within the <span class=\'code\'>MDataBlock</span>.</p>\n\
<blockquote>\n\
<p><strong>Important:</strong> A data block and its data are only valid during the execution of a node’s <span class=\'code\'>compute()</span> method. Never maintain a pointer to a data block or any of its data after the <span class=\'code\'>compute()</span> method exits.</p>\n\
</blockquote>\n\
<p>To access input data, use <span class=\'code\'>MDataBlock::inputValue()</span> to create an <span class=\'code\'>MDataHandle</span> object for the input attribute. Then extract the data using the appropriate <span class=\'code\'>MDataHandle</span> method.</p>\n\
<p>For example, to extract a float from the incoming data block, use the <span class=\'code\'>MDataHandle::asFloat()</span> method. In this example, <span class=\'code\'>input</span> is the input attribute.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">MStatus myNode::compute( const MPlug&amp; plug, MDataBlock&amp; datablock ) {\n\
\n\
    [...]\n\
    \n\
    MDataHandle inputHandle = datablock.inputValue(input, &amp;returnStatus);\n\
    float&amp; inputValue = inputHandle.asFloat();\n\
</pre></div><p>To update an output attribute, use <span class=\'code\'>MDataBlock::outputValue()</span> to create an <span class=\'code\'>MDataHandle</span> object for the output attribute. Set the output data using <span class=\'code\'>MDataHandle::set()</span>.</p>\n\
<p>In the following example, <span class=\'code\'>output</span> is the node&#39;s output attribute:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">MStatus myNode::compute( const MPlug&amp; plug, MDataBlock&amp; data ) {\n\
\n\
    [...]\n\
\n\
    float result = sin(inputValue);\n\
    MDataHandle outputHandle = data.outputValue(output);\n\
    outputHandle.set(result);\n\
    data.setClean(plug);\n\
</pre></div><p>After setting a new output attribute value, set its plug to clean using <span class=\'code\'>MDataBlock::setClean()</span> to indicate that it is newly computed and no longer dirty.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";