var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Communication Between Manipulators and Nodes</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Communication Between Manipulators and Nodes</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"communication-between-manipulators-and-nodes\"></a></div>\n\
<p>Manipulators communicate with plugs on nodes to set the values of the plugs. Additionally, manipulators also set the manipulator values with respect to the values of the plugs.</p>\n\
<p>The communication between manipulators and nodes is done in one of these two ways: simple one-to-one associations, or through complex conversion functions. The following diagram illustrates the communication between nodes and manipulators.</p>\n\
<div class=\'figure\'><img src=\'dev_help/images/comp_container.png\' title=\'\'></div>\n\
<p>The converter manages the communication between the plugs on the nodes and manipulator values. The arrows in the diagram indicate the direction in which the information flows. Each container manipulator has one converter, which is the interface between the children manipulators and the affected plugs.</p>\n\
<p>The square boxes indicate the data items on the converter and the base manipulators. The items on the converter that are related to the children manipulator values are called the converterManipValue items. The items on the converter that are related to the node plug values are called the converterPlugValue items.</p>\n\
<p>The items on the base manipulators are called the manipValue items. Some manipValue items relate directly to an affordance of the manipulator. For example, <span class=\'code\'>MFnDiscManip::angleIndex()</span> relates directly to the rotation affordance of the DiscManip. Other manipValue items do not relate to an affordance of the manipulator, but provide important information on the position or orientation of the manipulator such as <span class=\'code\'>MFnDiscManip::centerIndex()</span> and <span class=\'code\'>MFnDiscManip::axisIndex()</span>.</p>\n\
<p>Each converterManipValue item and converterPlugValue item has an integer index to uniquely identify the item. Each manipValue item on a base manipulator also has an integer index to uniquely identify the item.</p>\n\
<p>Note: There is a one-to-one correspondence between a converterManipValue item and a base manipulator’s manipValue item, and these items share the same integer index. There is also a one-to-one correspondence between a converterPlugValue item and a plug on a node, which is affected by the manipulator.</p>\n\
<p>As shown in the diagram, there are one-to-one associations directly between a converterManipValue item and a converterPlugValue item.</p>\n\
<p>More complex conversions between converterManipValue items and converterPlugValue items are performed through conversion functions. These functions can use any number of converterPlugValue items or converterManipValue items to calculate the corresponding converterManipValue or converterPlugValue item.</p>\n\
<div class=\'section\'><a id=\"one-to-one-association\"></a><h2 id=\"one-to-one-association\">One-to-one Association</h2></div>\n\
<p>One-to-one association synchronizes the manipulator values with the plug values. In other words, the manipulator value equals the plug value, and vice versa.</p>\n\
<p>One-to-one associations between a converterManipValue item and a converterPlugValue item are established through methods on the manipulator classes derived from <span class=\'code\'>MFnManip3D</span>.These methods and the data types corresponding to the plugs they connect to are the following:</p>\n\
<ul>\n\
<li><span class=\'code\'>MFnFreePointTriadManip::connectToPointPlug()</span> (three doubles)</li>\n\
<li><span class=\'code\'>MFnDirectionManip::connectToDirectionPlug()</span> (three doubles)</li>\n\
<li><span class=\'code\'>MFnDistanceManip::connectToDistancePlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnPointOnCurveManip::connectToCurvePlug()</span> (curve)</li>\n\
<li><span class=\'code\'>MFnPointOnCurveManip::connectToParamPlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnPointOnSurfaceManip::connectToSurfacePlug()</span> (surface)</li>\n\
<li><span class=\'code\'>MFnPointOnSurfaceManip::connectToParamPlug()</span> (two doubles)</li>\n\
<li><span class=\'code\'>MFnDiscManip::connectToAnglePlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnCircleSweepManip::connectToAnglePlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnToggleManip::connectToTogglePlug()</span> (boolean)</li>\n\
<li><span class=\'code\'>MFnStateManip::connectToStatePlug()</span> (long)</li>\n\
<li><span class=\'code\'>MFnCurveSegmentManip::connectToCurvePlug()</span> (curve)</li>\n\
<li><span class=\'code\'>MFnCurveSegmentManip::connectToStartParamPlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnCurveSegmentManip::connectToEndParamPlug()</span> (double)</li>\n\
<li><span class=\'code\'>MFnRotateManip::connectToRotatePlug()</span> (three doubles)</li>\n\
<li><span class=\'code\'>MFnRotateManip::connectToRotateCenterPlug()</span> (three doubles)</li>\n\
<li><span class=\'code\'>MFnScaleManip::connectToScalePlug()</span> (three doubles)</li>\n\
<li><span class=\'code\'>MFnScaleManip::connectToScaleCenterPlug()</span> (three doubles)</li>\n\
</ul>\n\
<p>These methods must be called from the <span class=\'code\'>connectToDependNode()</span> method. For example, in the footPrintManip example:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus footPrintLocatorManip::connectToDependNode(const MObject &amp;node)\n\
\n\
{\n\
    ...\n\
    MFnDistanceManip distanceManipFn(fDistanceManip);\n\
    MFnDependencyNode nodeFn(node);\n\
    MPlug sizePlug = nodeFn.findPlug(&quot;size&quot;, &amp;stat);\n\
    if (MStatus::kFailure != stat) {\n\
        distanceManipFn.connectToDistancePlug(sizePlug);\n\
        ...\n\
        finishAddingManips();\n\
        MPxManipContainer::connectToDependNode(node);\n\
    }\n\
    return stat;\n\
} \n\
\n\
</pre></div><p>The following figure shows the one-to-one association between the distance manipulator value and the value of the <em>size</em> plug on a node.</p>\n\
<div class=\'figure\'><img src=\'dev_help/images/mappingone.png\' title=\'\'></div>\n\
<p>After you set the one-to-one association, whenever the manipulator value changes, the corresponding plug value also changes, and vice versa.</p>\n\
<p>One-to-one association is a convenient approach to set the relationships between manipulator values and plug values. However, it might not be useful for more complex requirements. For example, you cannot set the distance manipulator value to 5 x the value of size. Also, when you move the node, the distance manipulator will not appear at the center of the node, and you cannot move the manipulator along the position of the node. If the position of a manipulator needs to be affected by the position of an object, or a group of manipulators has to move together in a specific way, then you need to use conversion functions.</p>\n\
<div class=\'section\'><a id=\"conversion-functions\"></a><h2 id=\"conversion-functions\">Conversion Functions</h2></div>\n\
<p>Conversion functions are used to convert between manipulator values and plug values. They are implemented as callback methods. Conversion functions are used to set complex relationships between manipulator values and plug values.</p>\n\
<p>A simple example of a manipulator that uses conversion functions is a container manipulator with a DiscManip connected to a plug (associated with an attribute of type <span class=\'code\'>MFnUnitAttribute::kAngle</span>) that takes the rotation of the disc manipulator and multiplies that rotation by 10. The conversion function uses <span class=\'code\'>MPxManipContainer::getConverterManipValue()</span> on <span class=\'code\'>MFnDiscManip::angleIndex()</span> and then multiplies that angle by 10.</p>\n\
<p>Conversion functions are very useful when the position of a manipulator has to be affected by the position of an object, or to move a group of manipulators together in a specific way. Without conversion functions, manipulators cannot move together as a unit, and certain components of the manipulator will remain either at the origin or a fixed position in space.</p>\n\
<p>Note: Conversion functions are not required when a FreePointTriadManip specifies a position or a PointOnCurveManip specifies a parameter along a curve. However, if you have a DiscManip that you want to move along with the PointOnCurveManip, you need a conversion function to give the DiscManip information about its position and normal.</p>\n\
<p>There are two kinds of conversion callback methods: plugToManip and manipToPlug.</p>\n\
<div class=\'section\'><a id=\"conversion-callback---plugtomanip\"></a><h3 id=\"conversion-callback---plugtomanip\">Conversion Callback - plugToManip</h3></div>\n\
<p>A plugToManip conversion callback is used to get the value of a converterManipValue item from various converterPlugValue items. This callback has access to all the converterPlugValue items and returns the value of a converterManipValue item.</p>\n\
<p>The plugToManip conversion is implemented by calling <span class=\'code\'>MPxManipContainer::addPlugToManipConversionCallback()</span>.</p>\n\
<p>Each visual control property has a corresponding index registered with the manipulator. To set the plugToManip conversion relationship, when <span class=\'code\'>MPxManipContainer::addPlugToManipConversionCallback()</span> is called, the index of the manipulator value must be passed. In each base manipulator function set class, there are functions to retrieve different indices for individual properties.</p>\n\
<p>The following code from footPrintManip illustrates how to retrieve the index of the starting point of the manipulator and pass it to the <span class=\'code\'>addPlugToManipConversionCallback()</span> function call. The second parameter of this function call is the callback conversion function, which calculates the manipulator value of the starting point based on some plug values.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">unsigned startPointIndex = distanceManipFn.startPointIndex();\n\
addPlugToManipConversionCallback (startPointIndex,(plugToManipConversionCallback)&amp;footPrintLocatorManip::startPointCallback);\n\
</pre></div><p>The plugToManip conversion callback accesses the plug values, calculates the manipulator values based on the custom algorithm, and returns them. In this example, the callback function does not request the plug values, instead, it retrieves the node translation in world space and returns this value. By doing so, it actually sets the start point manipulator value to be the same as the node translation. The result is that whenever the node moves, the manipulator moves along with it, and always appears in the center of the node.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MManipData footPrintLocatorManip::startPointCallback(unsigned index) const {\n\
    MFnNumericData numData;\n\
    MObject numDataObj =numData.create(MFnNumericData::k3Double);\n\
    MVector vec = nodeTranslation();\n\
    numData.setData (vec.x, vec.y, vec.z);\n\
    return(MManipData (numDataObj));\n\
}\n\
<div class=\'figure\'><img src=\'dev_help/images/plugToManip.png\' title=\'\'></div>\n\
</pre></div><p>The conversion callback returns a data type called MManipData and sets the manipulator value to what the MManipData represents.</p>\n\
<div class=\'section\'><a id=\"conversion-callback---maniptoplug\"></a><h3 id=\"conversion-callback---maniptoplug\">Conversion Callback - manipToPlug</h3></div>\n\
<p>A manipToPlug conversion callback is used to get the value of a converterPlugValue item from various converterManipValue items. This callback has access to all the converterManipValue items and returns the value of a converterPlugValue item.</p>\n\
<p>The manipToPlug conversion is implemented by calling <span class=\'code\'>MPxManipContainer::addManipToPlugConversionCallback()</span>. The corresponding plug needs to be passed in as the first parameter because the callback needs to know which plug values to change.</p>\n\
<p>The following code from the rotateManip example demonstrates using this technique with the node&#39;s rotate plug passed in.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MFnDependencyNode nodeFn (node);\n\
MPlug rPlug =nodeFn.findPlug (&quot;rotate&quot;, &amp;stat);\n\
…\n\
rotatePlugIndex = addManipToPlugConversionCallback(rPlug,(manipToPlugConversionCallback)&amp;exampleRotateManip::rotationChangedCallback);\n\
</pre></div><p>The <span class=\'code\'>addManipToPlugConversionCallback()</span> function call returns the index <span class=\'code\'>rotatePlugIndex</span> to identify which plug is registered with this callback. When the actual callback function is invoked, the index of the plug value to be calculated is passed in as the parameter of the function call. The callback function then determines if the passed-in value is the index that is registered for this callback. If there is more than one plug registered with this callback, a condition statement can be used to distinguish different passed-in plug index, and calculate corresponding manipulator values. In the following <span class=\'code\'>rotationChangedCallback()</span> example, the passed-in index is compared with <span class=\'code\'>rotatePlugIndex</span>. If they are equal, <span class=\'code\'>getConverterManipValue()</span> is called with rotate manipulator index to retrieve the rotate manipulator value. By returning the rotate manipulator value as a <span class=\'code\'>MManipData</span> object, this callback function is setting the rotate plug value to be the same as the rotate manipulator value. Depending on your specific requirement, more complex relationship between rotate plug value and rotate manipulator value can be set using conversion callback.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MManipData exampleRotateManip::rotationChangedCallback(unsigned index) {\n\
    MObject obj =MObject::kNullObj;\n\
    // If we entered the callback with an invalid index, print an error and\n\
    // return.  Because, we registered the callback only for one plug, all\n\
    // invocations of the callback must be for that plug.\n\
    if (index != rotatePlugIndex) {\n\
        MGlobal::displayError(&quot;Invalid index in rotation changed callback!&quot;);\n\
        // For invalid indices, return vector of 0&#39;s\n\
        MFnNumericData numericData;\n\
        obj = numericData.create(MFnNumericData::k3Double);\n\
        numericData.setData (0.0, 0.0, 0.0);\n\
        return (obj);\n\
    }\n\
    MFnNumericData numericData;\n\
    obj =numericData.create(MFnNumericData::k3Double); \n\
    MEulerRotation manipRotation;\n\
    if (!getConverterManipValue(rotateManip.rotationIndex(),manipRotation)) {\n\
        MGlobal::displayError(&quot;Error retrieving manip value&quot;);\n\
        numericData.setData(0.0, 0.0, 0.0);\n\
    } else {\n\
        numericData.setData(manipRotation.x, manipRotation.y, manipRotation.z);\n\
    }\n\
    return MManipData(obj);\n\
}\n\
<div class=\'figure\'><img src=\'dev_help/images/manipToPlug.png\' title=\'\'></div>\n\
</pre></div><p>In general, manipToPlug conversions are not used commonly. In addition to using converterPlugValues and converterManipValues, it is sometimes useful to use class data, such as a DAG path. (See the footPrintManip for an example of how fNodePath is used to calculate the node translation.) For manipulators that operate on components, it might also be useful to store initial component positions (see the componentScaleManip for an example of how this is done).</p>\n\
<div class=\'section\'><a id=\"mmanipdata\"></a><h3 id=\"mmanipdata\">MManipData</h3></div>\n\
<p>The conversion callback methods return a data type called <span class=\'code\'>MManipData</span>. <span class=\'code\'>MManipData</span> encapsulates manipulator data which is returned from the manipulator conversion functions. It represents data that is either simple or complex. The simple data methods on <span class=\'code\'>MManipData</span> are used to represent bool, short, long, unsigned, float, and double types.</p>\n\
<p>Note: Sometimes attributes associated with the simple data types have higher level meanings such as distance, angle, and time (for example, <span class=\'code\'>MFnUnitAttribute::kAngle</span>, <span class=\'code\'>MFnUnitAttribute::kDistance</span>, and <span class=\'code\'>MFnUnitAttribute::kTime</span>).</p>\n\
<p><span class=\'code\'>MManipData</span> is also used to represent complex data types created by <span class=\'code\'>MFnData</span> or classes derived from <span class=\'code\'>MFnData</span>, such as matrices, curves, and arrays of data.</p>\n\
<p>The footPrintManip plug-in has an example of a plugToManip conversion callback called startPointCallback. The startPointCallback returns an <span class=\'code\'>MManipData</span> which is set to an <span class=\'code\'>MObject</span> that is created by <span class=\'code\'>MFnNumericData</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">class footPrintLocatorManip : public MPxManipContainer\n\
{\n\
    public:\n\
        ...\n\
        MManipData startPointCallback(unsigned index) const;\n\
        MVector nodeTranslation() const;\n\
        MDagPath fDistanceManip;\n\
        ...\n\
};\n\
MManipData footPrintLocatorManip::startPointCallback\n\
    (unsigned index) const\n\
{\n\
    // The index is the startPointIndex that is\n\
    // specified in addPlugToManipConversionCallback,\n\
    // but it is not necessary to use this in the callback.\n\
    MFnNumericData numData;\n\
    MObject numDataObj =\n\
        numData.create(MFnNumericData::k3Double);\n\
    MVector vec = nodeTranslation();\n\
    numData.setData(vec.x, vec.y, vec.z);\n\
    return MManipData(numDataObj);\n\
}\n\
MStatus footPrintLocatorManip::connectToDependNode\n\
    (const MObject &amp;node)\n\
{\n\
    ...\n\
    unsigned startPointIndex =\n\
        distanceManipFn.startPointIndex();\n\
    addPlugToManipConversionCallback(\n\
        startPointIndex, \n\
        (plugToManipConversionCallback) startPointCallback);\n\
    ...\n\
}\n\
\n\
</pre></div><div class=\'section\'><a id=\"tips\"></a><h3 id=\"tips\">Tips</h3></div>\n\
<p><strong>Conversion Functions for Python and C++</strong> – For python plug-ins, the only functions that you can use to register conversion callback are: <span class=\'code\'>addManipToPlugConversion()</span> and <span class=\'code\'>addPlugToManipConversion()</span>. For C++ plug-ins, you can either use the <span class=\'code\'>addPlugToManipConversionCallback()</span>/<span class=\'code\'>addManipToPlugConversionCallback()</span> functions or <span class=\'code\'>addManipToPlugConversion()</span>/<span class=\'code\'>addPlugToManipConversion()</span> functions to register conversion callback.</p>\n\
<p><strong>Manip Index and Plug Index</strong> – The visual control properties have a manip index registered within the manipulator. For example, <span class=\'code\'>MFnDistanceManip::startPointIndex()</span> returns the index of the start point and <span class=\'code\'>MFnRotateManip::rotationIndex()</span> returns the index of the rotation manipulator value. To specify which manipulator value is set with a plugToManip conversion callback, the manip index must be used when <span class=\'code\'>addPlugToManipConversionCallback()</span> or <span class=\'code\'>addPlugToManipConversion()</span> is called.</p>\n\
<p>The plug index is returned by <span class=\'code\'>addManipToPlugConversionCallback()</span> and <span class=\'code\'>addManipToPlugConversion()</span> to identify which plug is registered with the manipToPlug conversion callback.</p>\n\
<p>In addition, the manip indexes are used to retrieve the manipulator values from the <span class=\'code\'>getConverterManipValue()</span> function, and the plug indexes are used to retrieve the plug values from the <span class=\'code\'>getConverterPlugValue()</span> function.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";