var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Error checking</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Error checking</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"error-checking\"></a></div>\n\
<p>In the examples presented so far you have not been prompted to do much error checking. This is usually fine for examples, but when producing a production plug-in you really want to check for errors.</p>\n\
<p>Most methods take an optional final argument, which is a pointer to an <a href=\'#!/url=./dev_help/Maya-API-introduction/MStatus-class.html\' title=\'\'><span class=\'code\'>MStatus</span></a> variable into which the status return value is put.</p>\n\
<p>If you replace the argument parsing code with the following fragment in the helix example, the example will be checking for, and handling, most possible errors.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint lang-cpp\">// Parse the arguments.\n\
for ( i = 0; i &lt; args.length(); i++ )\n\
    if ( MString( &quot;-p&quot; ) == args.asString( i, &amp;stat )\n\
        &amp;&amp; MS::kSuccess == stat )\n\
    {\n\
        double tmp = args.asDouble( ++i, &amp;stat );\n\
        // argument can be retrieved as a double\n\
        if ( MS::kSuccess == stat )\n\
            pitch = tmp;\n\
    }\n\
    else if ( MString( &quot;-r&quot; ) == args.asString( i, &amp;stat )\n\
        &amp;&amp; MS::kSuccess == stat )\n\
    {\n\
        double tmp = args.asDouble( ++i, &amp;stat );\n\
        // argument can be retrieved as a double\n\
        if ( MS::kSuccess == stat )\n\
            radius = tmp;\n\
    }\n\
\n\
</pre></div><p>The addition of <span class=\'code\'>&amp;stat</span> to the asString() and asDouble() methods allows you to check if the casting operation succeeds.</p>\n\
<p>For example, <span class=\'code\'>args.asString(i, &amp;stat)</span> could return <span class=\'code\'>MS::kFailure</span> if the index is greater than the number of arguments,</p>\n\
<p>or</p>\n\
<p><span class=\'code\'>args.asDouble(++i, &amp;stat)</span> could fail if the argument could not be converted to a double.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";