var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Querying the Scene Graph</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Querying the Scene Graph</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"querying-the-scene-graph\"></a></div>\n\
<p>This topic presents the basics of scene element organization in Maya.</p>\n\
<div class=\'section\'><a id=\"the-directed-acyclic-graph-dag\"></a><h2 id=\"the-directed-acyclic-graph-dag\">The Directed Acyclic Graph (DAG)</h2></div>\n\
<p>The scene graph in Maya is commonly referred to as the &quot;<strong>Directed Acyclic Graph</strong>&quot;, or <strong>DAG</strong>. The DAG is actually a subset of a much larger graph, known as the <em>Dependency Graph</em>, which encompasses a much wider variety of node types including shaders, deformers, constraints, etc. For more information, consult the next section, <a href=\'#!/url=./dev_help/Maya-Python-API/Dependency-Graph-Plug-in-Basics.html\' title=\'\'>Dependency Graph Plug-in Basics</a>. For now, we will focus our attention DAG nodes, which can be categorized in one of two ways:</p>\n\
<ol>\n\
<li><strong>Transform Nodes (<span class=\'code\'>MFn.kTransform</span>):</strong> This node type defines a local 4x4 transformation matrix which affects all the objects beneath it in the hierarchy. This transformation data is manipulated by the <span class=\'code\'>MFnTransform</span> function set. Transform nodes can have other transform nodes as their children to group scene elements together.</li>\n\
<li><strong>Shape Nodes (<span class=\'code\'>MFn.kMesh</span>, <span class=\'code\'>MFn.kCamera</span>, <span class=\'code\'>MFn.kLight</span>, ... ):</strong> This node type contains the actual geometric information of a scene element, such as the vertices of a mesh. A shape node always has a transform node as its parent.</li>\n\
</ol>\n\
<p>The diagram below presents the simplified directed acyclic graph (DAG) of a basic scene. The <span class=\'code\'>world</span> node represents the scene&#39;s root. The green circles correspond to the transform nodes (<span class=\'code\'>kTransform</span>), and allow the shapes to be positioned in the scene. The shape nodes are identified by blue circles. In this DAG, <span class=\'code\'>perspShape</span> corresponds to a camera in the scene, while <span class=\'code\'>pCubeShape1</span> and <span class=\'code\'>pointLightShape1</span> respectively represent a cubic mesh and a point light in the scene. Observe that the <span class=\'code\'>pointLight1</span> transform node is a child of the <span class=\'code\'>pCube1</span> transform node, which means that if the <span class=\'code\'>pCube1</span> transform node is moved, the point light will be moved as well.</p>\n\
<div class=\'figure\'><img src=\'images/MayaSceneGraph.png\' title=\'\'></div>\n\
<blockquote>\n\
<p><strong>Note:</strong> Generally, a shape node cannot have any children under it. The exception to this rule is a special circumstance known as the <em>underworld</em>. The <em>underworld</em> is a DAG subgraph whose root is attached as a child to a shape node. This <em>underworld</em> graph defines the control points and dependencies of a NURBS curve or a NURBS surface.</p>\n\
</blockquote>\n\
<div class=\'section\'><a id=\"dag-paths\"></a><h2 id=\"dag-paths\">DAG Paths</h2></div>\n\
<p>The location of a specific scene element within the DAG is identified by a <span class=\'code\'>MDagPath</span> object.</p>\n\
<p>The <span class=\'code\'>MDagPath</span> can return the <span class=\'code\'>MObject</span> to which it corresponds in the DAG using <span class=\'code\'>MDagPath.node()</span>. The path itself can be extended to encompass a shape node (<span class=\'code\'>MDagPath.extendToShape()</span>), or return the lowest transform node (<span class=\'code\'>MDagPath.transform()</span>) to name a few convenient functions. The <span class=\'code\'>MDagPath.fullPathName()</span> function returns a string representation of the DAG path to a given node, formatted as a sequence of pipe-separated (&quot;<span class=\'code\'>|</span>&quot;) node names starting at the nameless root of the DAG. In the diagram above, the string representation of the path from the root <span class=\'code\'>world</span> node to <span class=\'code\'>pointLightShape1</span> would be as follows: &quot;<span class=\'code\'>|pCube1|pointLight1|pointLightShape1</span>&quot; (note that the root node has no name).</p>\n\
<p>To reduce memory consumption, a complex shape such as a particularly dense mesh can be <em>instanced</em> in multiple locations in the scene graph. In other words, the same shape can appear in different places in the scene without being copied. To achieve this, several transform nodes in the DAG can be the parents of the same shape node. As such, a single shape node can have multiple paths from the root of the DAG. Functions such as <span class=\'code\'>MDagPath.isInstanced()</span>, and <span class=\'code\'>MDagPath.instanceNumber()</span> can be used to identify instanced shape nodes.</p>\n\
<div class=\'section\'><a id=\"traversing-the-scene-graph\"></a><h2 id=\"traversing-the-scene-graph\">Traversing the Scene Graph</h2></div>\n\
<p>The scene graph can be traversed using an <span class=\'code\'>MItDag</span> object, and optionally with an <span class=\'code\'>MItDependencyGraph</span> object, since the DAG is a subset of the Dependency Graph.</p>\n\
<p>Classes prefixed with <span class=\'code\'>MIt</span> are known as <em>iterators</em>, and allow you to inspect each object in a collection. In our case, <span class=\'code\'>MItDag</span> will allow us to iterate over the DAG nodes in the scene. In the following code sample, we create a DAG iterator which will traverse the scene starting at the root, and which will visit each node in a depth-first manner. The <span class=\'code\'>OpenMaya.MFn.kInvalid</span> parameter ensures that the <span class=\'code\'>MItDag</span> object will not filter any node types.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">dagIterator = OpenMaya.MItDag( OpenMaya.MItDag.kDepthFirst, OpenMaya.MFn.kInvalid )\n\
\n\
# This reference to the MFnDagNode function set will be needed\n\
# to obtain information about the DAG objects.\n\
dagNodeFn = OpenMaya.MFnDagNode()\n\
</pre></div><p>The <span class=\'code\'>MItDag.isDone()</span> function determines whether or not there are objects remaining to be inspected. The <span class=\'code\'>MItDag.currentItem()</span> function returns the iterator&#39;s current DAG object whose depth relative to the root can be obtained using <span class=\'code\'>MItDag.depth()</span>. Calling <span class=\'code\'>MItDag.next()</span> will cause the internal state of the iterator to advance to the next item, but will not return a DAG object; this is only achieved using <span class=\'code\'>MItDag.currentItem()</span>. If there are no more items to continue the iteration, the <span class=\'code\'>MItDag.isDone()</span> function will return <span class=\'code\'>True</span>. We can therefore construct a simple while loop to traverse the scene graph:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"># Traverse the scene.\n\
while( not dagIterator.isDone() ):\n\
\n\
    # Obtain the current item.\n\
    dagObject = dagIterator.currentItem()\n\
\n\
    # Extract the depth of the DAG object.\n\
    depth = dagIterator.depth()\n\
            \n\
    # Make our MFnDagNode function set operate on the current DAG object.\n\
    dagNodeFn.setObject( dagObject )\n\
                       \n\
    # Extract the DAG object&#39;s name.\n\
    name = dagNodeFn.name()\n\
            \n\
    print name + &#39; (&#39; + dagObject.apiTypeStr() + &#39;) depth: &#39; + str( depth )\n\
    \n\
    # Iterate to the next item.\n\
    dagIterator.next()\n\
<blockquote>\n\
</blockquote></pre></div><p><strong>Note:</strong> We elaborate on the use of <em>function sets</em> (ex: <span class=\'code\'>MFnDagNode</span>) in <a href=\'#!/url=./dev_help/Maya-Python-API/Command-Plug-in-Basics/Creating-and-Manipulating.html\' title=\'\'>Creating and Manipulating Objects</a>.</p>\n\
\n\
<div class=\'section\'><a id=\"inspecting-selected-scene-elements\"></a><h2 id=\"inspecting-selected-scene-elements\">Inspecting Selected Scene Elements</h2></div>\n\
<p>When objects are selected from Maya&#39;s user interface (or through scripts), they are added to a global <em>active selection list</em>, accessible via <span class=\'code\'>MGlobal.getActiveSelectionList()</span>. The <span class=\'code\'>MGlobal</span> static class provides a variety of functions pertaining to the Maya application, logging, object selection, command execution, 3D views (including the scene&#39;s up-axis), and model manipulation.</p>\n\
<p>The code below illustrates how to obtain the active selection list by populating a <span class=\'code\'>MSelectionList</span> object.</p>\n\
<p>Python API 2.0:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">selectionList = OpenMaya.MGlobal.getActiveSelectionList()\n\
</pre></div><p>Python API 1.0:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">selectionList = OpenMaya.MSelectionList()\n\
OpenMaya.MGlobal.getActiveSelectionList( selectionList )\n\
</pre></div><p>We use an instance of <span class=\'code\'>MItSelectionList</span> to iterate over the selection list. The constructor of <span class=\'code\'>MItSelectionList</span> allows us to specify a filter to iterate over objects of a specific type. In the following example, our iterator filters for objects compatible with the <span class=\'code\'>MFnDagNode</span> function set by specifying the <span class=\'code\'>MFn.kDagNode</span> parameter:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">iterator = OpenMaya.MItSelectionList( selectionList, OpenMaya.MFn.kDagNode )\n\
</pre></div><p>With the Python API 2.0, you can use a selection list to iterate over the entire scene graph by selecting all nodes in the scene first, and calling <span class=\'code\'>getDagPath()</span> on the iterator for each item. For example:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">dagNodeFn = OpenMaya.MFnDagNode()\n\
 \n\
cmds.select(all=True)\n\
selectionList = OpenMaya.MGlobal.getActiveSelectionList()\n\
if sList.length()&gt;0:\n\
    iterator = OpenMaya.MItSelectionList(selectionList, OpenMaya.MFn.kDagNode)\n\
    while not iterator.isDone():  \n\
        print iterator.getDagPath()\n\
        iterator.next()\n\
</pre></div><div class=\'section\'><a id=\"example-command-plug-in-printing-dag-paths\"></a><h2 id=\"example-command-plug-in-printing-dag-paths\">Example Command Plug-in: Printing DAG Paths</h2></div>\n\
<p><strong>Filename:</strong> <span class=\'code\'>printPaths.py</span></p>\n\
<p><strong>Sample Script Editor Output:</strong></p>\n\
<div class=\'figure\'><img src=\'images/printPaths.png\' title=\'\'></div>\n\
<p><strong>Program Summary:</strong> The plug-in code below creates the <span class=\'code\'>printPaths()</span> command. The behavior of this command depends on whether or not the <em>active selection list</em> contains DAG objects. If one or more DAG objects have been selected, their respective names, types, DAG paths and compatible function set types are printed to the Script Editor output. Otherwise, the scene graph is printed using each DAG node&#39;s name and type.</p>\n\
<p>Python API 2.0:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"># pyPrintPaths.py\n\
\n\
import sys\n\
import maya.cmds as cmds\n\
import maya.api.OpenMaya as OpenMaya\n\
\n\
def maya_useNewAPI():\n\
    &quot;&quot;&quot;\n\
    The presence of this function tells Maya that the plugin produces, and\n\
    expects to be passed, objects created using the Maya Python API 2.0.\n\
    &quot;&quot;&quot;\n\
    pass\n\
    \n\
kPluginCmdName = &#39;pyPrintPaths&#39;\n\
\n\
##########################################################\n\
# Plug-in \n\
##########################################################\n\
class printPathsCmd(OpenMaya.MPxCommand):\n\
    \n\
    def __init__(self):\n\
        &#39;&#39;&#39; Constructor. &#39;&#39;&#39;\n\
        OpenMaya.MPxCommand.__init__(self)\n\
        \n\
    def doIt(self, args):\n\
        &#39;&#39;&#39; \n\
        Print the DAG paths of the selected objects.\n\
        If no DAG objects are selected, print the entire\n\
        scene graph.\n\
        &#39;&#39;&#39;\n\
        \n\
        # Populate the MSelectionList with the currently selected\n\
        # objects using the static function MGlobal.getActiveSelectionList().\n\
        \n\
        #selectionList = OpenMaya.MSelectionList()\n\
        selectionList = OpenMaya.MGlobal.getActiveSelectionList()\n\
        \n\
        # This selection list can contain more than just scene elements (DAG nodes),\n\
        # so we must create an iterator over this selection list (MItSelectionList), \n\
        # and filter for objects compatible with the MFnDagNode function set (MFn.kDagNode).\n\
        iterator = OpenMaya.MItSelectionList( selectionList, OpenMaya.MFn.kDagNode )\n\
        \n\
        if iterator.isDone():\n\
            # Print the whole scene if there are no DAG nodes selected.\n\
            print &#39;=====================&#39;\n\
            print &#39; SCENE GRAPH (DAG):  &#39;\n\
            print &#39;=====================&#39;\n\
            self.printScene()\n\
        else:\n\
            # Print the paths of the selected DAG objects. \n\
            print &#39;=======================&#39;\n\
            print &#39; SELECTED DAG OBJECTS: &#39;\n\
            print &#39;=======================&#39;\n\
            self.printSelectedDAGPaths( iterator )\n\
    \n\
    def printSelectedDAGPaths(self, pSelectionListIterator):\n\
        &#39;&#39;&#39; Print the DAG path(s) of the selected object(s). &#39;&#39;&#39;\n\
        \n\
        # Create an MDagPath object which will be populated on each iteration.\n\
        dagPath = OpenMaya.MDagPath()\n\
        \n\
        # Obtain a reference to MFnDag function set to print the name of the DAG object\n\
        dagFn = OpenMaya.MFnDagNode()\n\
        \n\
        \n\
        \n\
        # Perform each iteration.\n\
        while( not pSelectionListIterator.isDone() ):\n\
            \n\
            # Populate our MDagPath object. This will likely provide\n\
            # us with a Transform node.\n\
            dagPath = pSelectionListIterator.getDagPath()\n\
            try:\n\
                # Attempt to extend the path to the shape node.\n\
                dagPath.extendToShape()\n\
            except Exception as e:\n\
                # Do nothing if this operation fails.\n\
                pass\n\
            \n\
            # Obtain the name of the object.\n\
            dagObject = dagPath.node()\n\
            dagFn.setObject( dagObject )\n\
            name = dagFn.name()\n\
            \n\
            # Obtain the compatible function sets for this DAG object.\n\
            # These values refer to the enumeration values of MFn\n\
            fntypes = []\n\
            fntypes = OpenMaya.MGlobal.getFunctionSetList( dagObject )\n\
            \n\
            # Print the DAG object information.\n\
            print name + &#39; (&#39; + dagObject.apiTypeStr + &#39;)&#39;\n\
            print &#39;\\tDAG path: [&#39; + str( dagPath.fullPathName() ) + &#39;]&#39;\n\
            print &#39;\\tCompatible function sets: &#39; + str( fntypes )\n\
            \n\
            # Advance to the next item\n\
            pSelectionListIterator.next()\n\
        \n\
        print &#39;=====================&#39;         \n\
            \n\
    def printScene(self):\n\
        &#39;&#39;&#39; Traverse and print the elements in the scene graph (DAG)  &#39;&#39;&#39;\n\
        \n\
        # Create a function set which we will re-use throughout our scene graph traversal.\n\
        dagNodeFn = OpenMaya.MFnDagNode()\n\
        \n\
        # Create an iterator to traverse the scene graph starting at the world node\n\
        # (the scene&#39;s origin). We use a depth-first traversal, and we do not filter for\n\
        # any scene elements, as indicated by the &#39;OpenMaya.MFn.kInvalid&#39; parameter.\n\
        dagIterator = OpenMaya.MItDag( OpenMaya.MItDag.kDepthFirst,\n\
                                       OpenMaya.MFn.kInvalid )\n\
       \n\
        # Traverse the scene.\n\
        while( not dagIterator.isDone() ):\n\
            \n\
            # Obtain the current item.\n\
            dagObject = dagIterator.currentItem()\n\
            depth = dagIterator.depth()\n\
            \n\
            # Make our MFnDagNode function set operate on the current DAG object.\n\
            dagNodeFn.setObject( dagObject )\n\
                       \n\
            # Extract the DAG object&#39;s name.\n\
            name = dagNodeFn.name()\n\
            \n\
            # Generate our output by first incrementing the tabs based on the depth\n\
            # of the current object. This formats our output nicely.\n\
            output = &#39;&#39;\n\
            for i in range( 0, depth ):\n\
                output += &#39;\\t&#39;\n\
                \n\
            output += name + &#39; (&#39; + dagObject.apiTypeStr + &#39;)&#39;\n\
            print output\n\
            \n\
            # Increment to the next item.\n\
            dagIterator.next()\n\
        \n\
        print &#39;=====================&#39;\n\
\n\
    \n\
\n\
\n\
##########################################################\n\
# Plug-in initialization.\n\
##########################################################       \n\
def cmdCreator():\n\
    &#39;&#39;&#39; Creates an instance of our command class. &#39;&#39;&#39;\n\
    return printPathsCmd() \n\
    \n\
def initializePlugin(mobject):\n\
    &#39;&#39;&#39; Initializes the plug-in.&#39;&#39;&#39;\n\
    mplugin = OpenMaya.MFnPlugin( mobject )\n\
    try:\n\
        mplugin.registerCommand( kPluginCmdName, cmdCreator )\n\
    except:\n\
        sys.stderr.write( &quot;Failed to register command: %s\\n&quot; % kPluginCmdName )\n\
\n\
def uninitializePlugin(mobject):\n\
    &#39;&#39;&#39; Uninitializes the plug-in &#39;&#39;&#39;\n\
    mplugin = OpenMaya.MFnPlugin( mobject )\n\
    try:\n\
        mplugin.deregisterCommand( kPluginCmdName )\n\
    except:\n\
        sys.stderr.write( &quot;Failed to unregister command: %s\\n&quot; % kPluginCmdName )\n\
\n\
##########################################################\n\
# Sample usage.\n\
##########################################################\n\
&#39;&#39;&#39;\n\
# Copy the following lines and run them in Maya&#39;s Python Script Editor:\n\
\n\
import maya.cmds as cmds\n\
cmds.loadPlugin( &#39;pyPrintPaths.py&#39; )\n\
cmds.pyPrintPaths()\n\
 \n\
&#39;&#39;&#39;\n\
</pre></div><p>Python API 1.0:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"># printPaths.py\n\
\n\
import sys\n\
import maya.OpenMayaMPx as OpenMayaMPx\n\
import maya.OpenMaya as OpenMaya\n\
\n\
kPluginCmdName = &#39;printPaths&#39;\n\
\n\
##########################################################\n\
# Plug-in \n\
##########################################################\n\
class printPathsCmd(OpenMayaMPx.MPxCommand):\n\
    \n\
    def __init__(self):\n\
        &#39;&#39;&#39; Constructor. &#39;&#39;&#39;\n\
        OpenMayaMPx.MPxCommand.__init__(self)\n\
        \n\
    def doIt(self, args):\n\
        &#39;&#39;&#39; \n\
        Print the DAG paths of the selected objects.\n\
        If no DAG objects are selected, print the entire\n\
        scene graph.\n\
        &#39;&#39;&#39;\n\
        \n\
        # Populate the MSelectionList with the currently selected\n\
        # objects using the static function MGlobal.getActiveSelectionList().\n\
        selectionList = OpenMaya.MSelectionList()\n\
        OpenMaya.MGlobal.getActiveSelectionList( selectionList )\n\
        \n\
        # This selection list can contain more than just scene elements (DAG nodes),\n\
        # so we must create an iterator over this selection list (MItSelectionList), \n\
        # and filter for objects compatible with the MFnDagNode function set (MFn.kDagNode).\n\
        iterator = OpenMaya.MItSelectionList( selectionList, OpenMaya.MFn.kDagNode )\n\
        \n\
        if iterator.isDone():\n\
            # Print the whole scene if there are no DAG nodes selected.\n\
            self.printScene()\n\
        else:\n\
            # Print the paths of the selected DAG objects. \n\
            self.printSelectedDAGPaths( iterator )\n\
    \n\
    def printSelectedDAGPaths(self, pSelectionListIterator):\n\
        &#39;&#39;&#39; Print the DAG path(s) of the selected object(s). &#39;&#39;&#39;\n\
        \n\
        # Create an MDagPath object which will be populated on each iteration.\n\
        dagPath = OpenMaya.MDagPath()\n\
        \n\
        # Obtain a reference to MFnDag function set to print the name of the DAG object\n\
        dagFn = OpenMaya.MFnDagNode()\n\
        \n\
        print &#39;=======================&#39;\n\
        print &#39; SELECTED DAG OBJECTS: &#39;\n\
        print &#39;=======================&#39;\n\
        \n\
        # Perform each iteration.\n\
        while( not pSelectionListIterator.isDone() ):\n\
            \n\
            # Populate our MDagPath object. This will likely provide\n\
            # us with a Transform node.\n\
            pSelectionListIterator.getDagPath( dagPath )\n\
            try:\n\
                # Attempt to extend the path to the shape node.\n\
                dagPath.extendToShape()\n\
            except Exception as e:\n\
                # Do nothing if this operation fails.\n\
                pass\n\
            \n\
            # Obtain the name of the object.\n\
            dagObject = dagPath.node()\n\
            dagFn.setObject( dagObject )\n\
            name = dagFn.name()\n\
            \n\
            # Obtain the compatible function sets for this DAG object.\n\
            # These values refer to the enumeration values of MFn\n\
            fntypes = []\n\
            OpenMaya.MGlobal.getFunctionSetList( dagObject, fntypes )\n\
            \n\
            # Print the DAG object information.\n\
            print name + &#39; (&#39; + dagObject.apiTypeStr() + &#39;)&#39;\n\
            print &#39;\\tDAG path: [&#39; + str( dagPath.fullPathName() ) + &#39;]&#39;\n\
            print &#39;\\tCompatible function sets: &#39; + str( fntypes )\n\
            \n\
            # Advance to the next item\n\
            pSelectionListIterator.next()\n\
        \n\
        print &#39;=====================&#39;\n\
            \n\
            \n\
    def printScene(self):\n\
        &#39;&#39;&#39; Traverse and print the elements in the scene graph (DAG)  &#39;&#39;&#39;\n\
        \n\
        # Create a function set which we will re-use throughout our scene graph traversal.\n\
        dagNodeFn = OpenMaya.MFnDagNode()\n\
        \n\
        # Create an iterator to traverse the scene graph starting at the world node\n\
        # (the scene&#39;s origin). We use a depth-first traversal, and we do not filter for\n\
        # any scene elements, as indicated by the &#39;OpenMaya.MFn.kInvalid&#39; parameter.\n\
        dagIterator = OpenMaya.MItDag( OpenMaya.MItDag.kDepthFirst,\n\
                                       OpenMaya.MFn.kInvalid )\n\
\n\
        print &#39;=====================&#39;\n\
        print &#39; SCENE GRAPH (DAG):  &#39;\n\
        print &#39;=====================&#39;\n\
        \n\
        # Traverse the scene.\n\
        while( not dagIterator.isDone() ):\n\
            \n\
            # Obtain the current item.\n\
            dagObject = dagIterator.currentItem()\n\
            depth = dagIterator.depth()\n\
            \n\
            # Make our MFnDagNode function set operate on the current DAG object.\n\
            dagNodeFn.setObject( dagObject )\n\
                       \n\
            # Extract the DAG object&#39;s name.\n\
            name = dagNodeFn.name()\n\
            \n\
            # Generate our output by first incrementing the tabs based on the depth\n\
            # of the current object. This formats our output nicely.\n\
            output = &#39;&#39;\n\
            for i in range( 0, depth ):\n\
                output += &#39;\\t&#39;\n\
                \n\
            output += name + &#39; (&#39; + dagObject.apiTypeStr() + &#39;)&#39;\n\
            print output\n\
            \n\
            # Increment to the next item.\n\
            dagIterator.next()\n\
        \n\
        print &#39;=====================&#39;\n\
\n\
\n\
##########################################################\n\
# Plug-in initialization.\n\
##########################################################       \n\
def cmdCreator():\n\
    &#39;&#39;&#39; Creates an instance of our command class. &#39;&#39;&#39;\n\
    return OpenMayaMPx.asMPxPtr( printPathsCmd() )\n\
    \n\
def initializePlugin(mobject):\n\
    &#39;&#39;&#39; Initializes the plug-in.&#39;&#39;&#39;\n\
    mplugin = OpenMayaMPx.MFnPlugin( mobject )\n\
    try:\n\
        mplugin.registerCommand( kPluginCmdName, cmdCreator )\n\
    except:\n\
        sys.stderr.write( &quot;Failed to register command: %s\\n&quot; % kPluginCmdName )\n\
\n\
def uninitializePlugin(mobject):\n\
    &#39;&#39;&#39; Uninitializes the plug-in &#39;&#39;&#39;\n\
    mplugin = OpenMayaMPx.MFnPlugin( mobject )\n\
    try:\n\
        mplugin.deregisterCommand( kPluginCmdName )\n\
    except:\n\
        sys.stderr.write( &quot;Failed to unregister command: %s\\n&quot; % kPluginCmdName )\n\
\n\
##########################################################\n\
# Sample usage.\n\
##########################################################\n\
&#39;&#39;&#39;\n\
# Copy the following lines and run them in Maya&#39;s Python Script Editor:\n\
\n\
import maya.cmds as cmds\n\
cmds.loadPlugin( &#39;printPaths.py&#39; )\n\
cmds.printPaths()\n\
 \n\
&#39;&#39;&#39;\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";