var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Components</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Components</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"components\"></a></div>\n\
<p>Shapes which require visual feedback and manipulation of attributes, such as control point based shapes, must have components associated with those attributes. Shapes such as polygonal meshes or spline surfaces have control vertices which can be selected and manipulated. These control vertices exist as attributes on the shape which are exposed interactively in Maya by representing them as components.</p>\n\
<p>Components are objects (<span class=\'code\'>MObject</span>s) which contain two pieces of information, the type of component and the index values or range. An example is a vertex component for a mesh shape where <span class=\'code\'>vtx[0]</span> represents vertex 0 of the mesh and <span class=\'code\'>vtx[0:7]</span> represents the first 8 vertices of the mesh.</p>\n\
<p>The classes used for creating, editing, and querying components are:</p>\n\
<ul>\n\
<li><span class=\'code\'>MFnComponent</span></li>\n\
<li><span class=\'code\'>MFnDoubleIndexedComponent</span></li>\n\
<li><span class=\'code\'>MFnSingleIndexedComponent</span></li>\n\
<li><span class=\'code\'>MFnTripleIndexedComponent</span></li>\n\
</ul>\n\
<p>Components fall into three categories based upon the dimensions of the index. The types are single, double, and triple indexed. Examples of these types are mesh vertices (single indexed), NURBS surface CVs (double indexed), and lattice points (triple indexed).</p>\n\
<p>Components can be marked as complete, meaning the component represents a complete set of indices from 0 to numElements-1.</p>\n\
<div class=\'section\'><a id=\"writing-a-component-shape\"></a><h2 id=\"writing-a-component-shape\">Writing a Component Shape</h2></div>\n\
<p>If you want to create a surface shape that supports components, the <span class=\'code\'>MPxComponentShape</span> class is a better starting point than the <span class=\'code\'>MPxSurfaceShape</span> class. <span class=\'code\'>MPxComponentShape</span> is derived from <span class=\'code\'>MPxSurfaceShape</span> and provides the basic functionality for users to work with shapes that support components.</p>\n\
<p>If you need to edit the user interface aspects of the shape, derive from the <span class=\'code\'>MPxSurfaceShapeUI</span> class. This class allows drawing and interactive selection of any components in a shape.</p>\n\
<div class=\'section\'><a id=\"mapping-attributes-to-components\"></a><h2 id=\"mapping-attributes-to-components\">Mapping attributes to components</h2></div>\n\
<p>In Maya, components are specified as strings. Each type of component has a different string name. In the API, components are <span class=\'code\'>MObjects</span> distinguished by their API type (see <span class=\'code\'>MFn.h</span>). For example, a mesh vertex component can be specified in Maya as <span class=\'code\'>vtx[0]</span>, and in a plug-in it is represented as an <span class=\'code\'>MObject</span> with <span class=\'code\'>apiType</span> <span class=\'code\'>MFn::kMeshVertComponent</span>. The index information can be extracted using an <span class=\'code\'>MFnComponent</span> derived class.</p>\n\
<p>To associate (or map) a component with one of your shapes attributes, you must choose one of Maya’s existing component types and override <span class=\'code\'>MPxSurfaceShape::componentToPlugs</span> to convert component types to plugs.</p>\n\
<p>The following is an example of associating a mesh vertex component with the <span class=\'code\'>mControlPoints</span> attribute of a shape:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void yourShape::componentToPlugs( MObject&amp; component,\n\
                 MSelectionList&amp; list )const\n\
{\n\
     if ( component.hasFn(MFn::kMeshVertComponent) ) {\n\
         MFnSingleIndexedComponent fnVtxComp( component );\n\
         MObject thisNode = thisMObject();\n\
         MPlug plug( thisNode, mControlPoints );\n\
         int len = fnVtxComp.elementCount();\n\
         for ( int i = 0; i &lt; len; i++ ) {\n\
             MPlug vtxPlug = plug.elementByLogicalIndex(\n\
             fnVtxComp.element(i) );\n\
         list.add( vtxPlug );\n\
         }\n\
     }\n\
}\n\
\n\
</pre></div><div class=\'section\'><a id=\"component-matching\"></a><h2 id=\"component-matching\">Component matching</h2></div>\n\
<p>Attributes can be specified as strings in MEL. Your shape must be able to validate these strings to ensure that proper names, indices etc. have been given. The method <span class=\'code\'>MPxSurfaceShape::matchComponent</span> is used for this purpose.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">virtual MatchResult matchComponent( const MSelectionList&amp; item,\n\
                 const MAttributeSpecArray&amp; spec,\n\
                 MSelectionList&amp; list );\n\
\n\
</pre></div><p>This method validates component names and indices which are specified as a string and adds the corresponding component to the passed in selection list. Select commands such as <span class=\'code\'>select shape1.vtx[0:7]</span> are validated with this method and the corresponding component is added to the selection list.</p>\n\
<p>The attribute specification (<span class=\'code\'>MAttributeSpec</span>) is a class that provides convenient access to all of the information about how attributes are specified. This includes attribute names, indices, and ranges.</p>\n\
<div class=\'section\'><a id=\"component-iteration\"></a><h2 id=\"component-iteration\">Component iteration</h2></div>\n\
<p>For Maya to get and set the position of components you must define an iterator for your geometry by deriving from the class <span class=\'code\'>MPxGeometryIterator</span>.</p>\n\
<p>A geometry iterator is used by the translate/rotate/scale manipulators to determine where to place the manipulator when components are selected.</p>\n\
<p>Deformers also require a geometry iterator with overridden setPoint and point methods in order to deform the points your shape.</p>\n\
<p>In general, you will want to override the following methods from <span class=\'code\'>MPxGeometryIterator</span>:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"> MPxGeometryIterator( void * userGeometry,\n\
 MObjectArray &amp; components );\n\
 MPxGeometryIterator( void * userGeometry,\n\
 MObject &amp; components );\n\
 virtual void reset();\n\
 virtual MPoint point() const;\n\
 virtual void setPoint( const MPoint &amp; ) const;\n\
\n\
</pre></div><p>You must override the following functions of <span class=\'code\'>MPxSurfaceShape</span> to associated an iterator with your shape:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"> virtual MPxGeometryIterator* \n\
 geometryIteratorSetup( MObjectArray&amp;, MObject&amp;, bool );\n\
 virtual bool acceptsGeometryIterator( bool writeable );\n\
 virtual bool acceptsGeometryIterator( MObject&amp;,\n\
 bool, bool );\n\
\n\
</pre></div><div class=\'section\'><a id=\"translate-scale-and-rotate-tools-for-components\"></a><h2 id=\"translate-scale-and-rotate-tools-for-components\">Translate, scale, and rotate tools for components</h2></div>\n\
<p>To support the translate, rotate and scale tools, you must override the method <span class=\'code\'>MPxSurfaceShape::transformUsing</span>. The function takes a matrix and array of components as arguments. The matrix specifies the transformation that is being applied and the components specify the attribute indices that are being transformed.</p>\n\
<p>For shapes with large numbers of control vertices, it can be prohibitively slow relying on Maya’s compute mechanism for setting attribute values. The method <span class=\'code\'>MPxNode::forceCache</span> can be used in these situations to gain direct access to a node’s datablock and to get/set attribute values directly without going through compute. Special care must be taken to ensure that all attributes that depend on the ones you are changing also get updated. For instance, if the vertices of a mesh are changed, then the normals should also be updated as well as the bounding box. The method, <span class=\'code\'>vertexOffsetDirection</span>, must be overridden if the Move tool is to work in move normal mode.</p>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";