var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Drawing and refresh</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Drawing and refresh</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"drawing-and-refresh\"></a></div>\n\
<p>Drawing is a two step process. In the first, the geometry and materials are evaluated and all of the information necessary for drawing is placed onto a queue. In the second, the actual OpenGL drawing occurs. This two step process allows your shapes to take advantage of the multi-threaded drawing capabilities of Maya.</p>\n\
<p>Drawing occurs whenever the camera changes or the view has to be refreshed. When this happens, the <span class=\'code\'>MPxSurfaceShapeUI::getDrawRequest</span> function is called. This is Maya’s way of asking the shape what it needs to draw. Inside this function you should query the drawing state, using <span class=\'code\'>MDrawInfo</span>, and then construct a draw request (<span class=\'code\'>MDrawRequest</span>) to place on the queue. You will often want to place multiple requests on the queue (<span class=\'code\'>MDrawRequestQueue</span>) in this function. For instance, in shaded mode if your shape is selected you may want to add a request to draw the shaded object and another request to draw wireframe on top of the shaded object.</p>\n\
<p>The draw data (<span class=\'code\'>MDrawData</span>) holds information specific to your shape which Maya does not intrinsically know about. The draw data acts as a container to pass your geometry through the draw request queue. For each draw request you must create and add a draw data object which contains geometry-specific information that you will need in the subsequent call to <span class=\'code\'>MPxSurfaceShapeUI::draw</span>.</p>\n\
<p>To create draw data, use the function <span class=\'code\'>MPxSurfaceShapeUI::getDrawData</span> and to add the data to a request use <span class=\'code\'>MDrawRequest::setDrawData</span>. The following example shows how to create a draw request and draw data for your geometry.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void yourShapeUI::getDrawRequests( const MDrawInfo &amp; info,\n\
 bool objectAndActiveOnly,\n\
 MDrawRequestQueue &amp; queue )\n\
{\n\
    MDrawData data;\n\
    MDrawRequest request = info.getPrototype( *this );\n\
    yourShape* shapeNode = (yourShape*)surfaceShape();\n\
    yourGeom* geom = shapeNode-&gt;geometry();\n\
    getDrawData( geom, data );\n\
    request.setDrawData( data );\n\
    ...\n\
}\n\
\n\
</pre></div><p>The draw request (<span class=\'code\'>MDrawRequest</span>) should be created in the overridden <span class=\'code\'>MPxSurfaceShapeUI::getDrawRequests</span> method. Once the request is created the appropriate &quot;set&quot; methods of this class should be used to define what is being requested. Then the request should be placed on the draw request queue using <span class=\'code\'>MDrawRequestQueue::add</span>.</p>\n\
<p>The draw token is an integer value which you can use to specify what is to be drawn. This is object specific and so you should define an enum with the information you require to decide what is being drawn in your <span class=\'code\'>MPxSurfaceShapeUI::draw</span> method. Here is an example of a draw token for a mesh object:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"> enum {\n\
          kDrawVertices, // component token\n\
          kDrawWireframe,\n\
          kDrawWireframeOnShaded,\n\
          kDrawSmoothShaded,\n\
          kDrawFlatShaded,\n\
          kLastToken\n\
      };\n\
\n\
</pre></div><p>Maya processes the draw request queue and for each draw request, calls the corresponding objects draw function. In the case of user defined shapes, your <span class=\'code\'>MPxSurfaceShapeUI::draw</span> method is called.</p>\n\
<div class=\'section\'><a id=\"drawing-in-shaded-mode\"></a><h2 id=\"drawing-in-shaded-mode\">Drawing in shaded mode</h2></div>\n\
<p>Supporting shaded mode display is a two step process. The first step occurs in your <span class=\'code\'>getDrawRequests</span> function. Here you must &quot;evaluate&quot; or setup the material so that it can be displayed when your object is drawn. The second step occurs in your draw function where you must setup the view to display the material.</p>\n\
<p>The following code demonstrates how to setup the material in your <span class=\'code\'>getDrawRequests</span> function if the request is for shaded mode display.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\"> MDagPath path = info.multiPath();\n\
 M3dView view = info.view();\n\
 MMaterial material = MPxSurfaceShapeUI::material( path );\n\
 material.evaluateMaterial( view, path );\n\
 if ( material.materialIsTextured() ) {\n\
     material.evaluateTexture( data );\n\
 }\n\
 request.setMaterial( material );\n\
\n\
</pre></div><p>In your draw function you will need to setup the scene view so that it can display the material. This is done using <span class=\'code\'>MMaterial::setMaterial</span>. To set up the scene view to display textures, use <span class=\'code\'>MMaterial::applyTexture</span>. The following code demonstrates this.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void yourShapeUI::draw( const MDrawRequest&amp; request,\n\
 M3dView &amp; view ) const\n\
{\n\
     ...\n\
     MMaterial material = request.material();\n\
     material.setMaterial( request.isTransparent() );\n\
     drawTexture = material.materialIsTextured();\n\
     if ( drawTexture ) glEnable(GL_TEXTURE_2D);\n\
     if ( drawTexture ) {\n\
         material.applyTexture( view, data );\n\
     }\n\
     ...\n\
}\n\
\n\
<blockquote>\n\
</blockquote></pre></div><p><strong>Note:</strong> All OpenGL calls should be enclosed by calls to <span class=\'code\'>M3dView::beginGL()</span> and <span class=\'code\'>M3dView::endGL()</span>.</p>\n\
\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";