var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Manipulators</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Manipulators</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"manipulators\"></a></div>\n\
<p>Manipulators (<span class=\'code\'>MPxManipulatorNode</span>, and <span class=\'code\'>MPxManipContainer</span>) are drawn in the old viewport using the following interface:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual void draw(M3dView &amp; view, const MDagPath &amp; path, M3dView::DisplayStyle style,M3dView::DisplayStatus status);</span></li>\n\
</ul>\n\
<p>The Viewport 2.0 interfaces attempt to match the old interface signatures, but branch off to allow access to an <span class=\'code\'>MUIDrawManager</span> and a frame context (<span class=\'code\'>MFrameContext</span>):</p>\n\
<ul>\n\
<li>Data Caching: <span class=\'code\'>virtual void preDrawUI( const M3dView &amp;view );</span></li>\n\
<li>Data Draw: <span class=\'code\'>virtual void drawUI( MHWRender::MUIDrawManager&amp; drawManager, const MHWRender::MFrameContext&amp; frameContext) const;</span></li>\n\
</ul>\n\
<p>One key difference between Viewport 2.0 and the old viewport is that an <span class=\'code\'>M3dView</span> (3d viewport wrapper) is no longer available for use at draw time. The equivalent state information is to be extracted from the frame context.</p>\n\
<p>Sample code from the example plug-in <em>footPrintLocatorManip</em> is shown below:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void footPrintLocatorManip::preDrawUI( const M3dView &amp;view )\n\
{\n\
   // Update text drawing position\n\
   fTextPosition = // DG evaluation\n\
}\n\
\n\
void footPrintLocatorManip::drawUI(\n\
MHWRender::MUIDrawManager&amp; drawManager,\n\
const MHWRender::MFrameContext&amp; frameContext ) const\n\
{\n\
// Draw some text\n\
    drawManager.beginDrawable();\n\
    drawManager.setColor( MColor( 0.0f, 1.0f, 0.1f ) );\n\
    drawManager.text( fTextPosition, “Some text&quot;, MHWRender::MUIDrawManager::kLeft );\n\
    drawManager.text2d( MPoint(100,100), “Some text in 2D&quot;, MHWRender::MUIDrawManager::kLeft );\n\
    drawManager.endDrawable();\n\
}\n\
</pre></div><p>Here, a text position is computed and stored in <span class=\'code\'>fTextPosition</span>. This value is then used at draw time to set the position to draw text using the <span class=\'code\'>MUIDrawManager</span> instance.</p>\n\
<p>For selection, it is recommended that you have a series of manipulators which are part of a <span class=\'code\'>MPxManipContainer</span>. If the plug-in wishes to select a sub-part or <em>handle</em>, then the <span class=\'code\'>void MUIDrawManager::beginDrawable(unsigned int name, bool nameIsPickable);</span> method can be used. The <em>name</em> argument is device independent and can be used with the <span class=\'code\'>MPxManipulatorNode::glActiveName()</span> method. The name is only used for selection and is be ignored in the regular draw pass.</p>\n\
<p>The <span class=\'code\'>lineManip</span> plug-in shows a sample usage:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">void    lineManip::drawUI( \n\
    MHWRender::MUIDrawManager&amp; drawManager, \n\
    const MHWRender::MFrameContext&amp; frameContext ) const\n\
{\n\
    // Find out if we should draw using selected color\n\
    bool drawAsSelected = false;\n\
    shouldDrawHandleAsSelected(lineName, drawAsSelected);\n\
    \n\
    // Everything from begin/endDrawable has the lineName handle and is considered to be pickable\n\
    drawManager.beginDrawable(lineName, true);\n\
    drawManager.setColorIndex( drawAsSelected ? fSelectedLineColorIndex : fLineColorIndex );\n\
    drawManager.line( fLineStart, fLineEnd );\n\
    drawManager.endDrawable();\n\
\n\
    // The text is not considered to be pickable\n\
    drawManager.beginDrawable();\n\
    drawManager.setColorIndex( fLineColorIndex );\n\
    drawManager.text( fLineStart, MString(&quot;line manip&quot;));\n\
    drawManager.endDrawable();\n\
\n\
    // The 2d line is considered to be pickable and has the same handle.\n\
    drawManager.beginDrawable(lineName, true);\n\
    drawManager.setColorIndex( drawAsSelected ? fSelectedLineColorIndex : fLineColorIndex );\n\
    drawManager.line2d(MPoint(100, 100), MPoint(200, 100));\n\
    drawManager.setLineWidth(5.0f);\n\
    drawManager.endDrawable();\n\
\n\
    // The 2d text is not considered to be pickable\n\
    drawManager.beginDrawable();\n\
    drawManager.setColorIndex( fLineColorIndex );\n\
    drawManager.setLineWidth(5.0f);\n\
    drawManager.text2d(MPoint(100, 105), MString(&quot;line manip 2D&quot;));\n\
    drawManager.endDrawable();\n\
}\n\
</pre></div>      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";