var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>UI draw manager</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>UI draw manager</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"ui-draw-manager\"></a></div>\n\
<p>The <span class=\'code\'>MUIDrawManager</span> class provides a straight forward way for API users to draw basic UI elements such as: lines, circles, arcs, icons, and text.</p>\n\
<p>While previous APIs allowed for what is commonly called <em>immediate mode</em> drawing, <span class=\'code\'>MUIDrawManager</span> exposes a different type of interaction closer to <em>retained mode</em> drawing. The logic consists of queuing items to draw “drawables” into a “drawable” list.</p>\n\
<p>Actual retained mode drawing is associated with <span class=\'code\'>MRenderItem</span> usage. You should use interfaces that use <span class=\'code\'>MRenderItem</span>s when more complex geometry or shading is required, and <span class=\'code\'>MUIDrawManager</span> for additional drawables.</p>\n\
<p>The decision as to whether to use <span class=\'code\'>MUIDrawManager</span> is mostly based on a choice between simplicity versus flexibility. A basic breakdown of the pros and cons of using <span class=\'code\'>MUIDrawManager</span> is provided in the following table:</p>\n\
<table cellpadding=\"0\" cellspacing=\"0\" class=\"not-ruled\">\n\
<colgroup>\n\
<col>\n\
<col>\n\
</colgroup>\n\
<tbody>\n\
<tr class=\"not-ruled-heading\">\n\
<th class=\"table-heading\">\n\
Strengths\n\
</th>\n\
<th class=\"table-heading\">\n\
Caveats\n\
</th>\n\
</tr>\n\
<tr class=\"not-ruled-odd-row\">\n\
<td class=\"table-body\">\n\
Suitable for simple UI, where simple means a small amount of UI, or a when a small number of objects are drawing the UI.\n\
</td>\n\
<td class=\"table-body\">\n\
May not scale well since drawables are not persistent, though <span class=\"code\">MPxSubSceneOverride</span> allows for reuse.\n\
</td>\n\
</tr>\n\
<tr class=\"not-ruled-even-row\">\n\
<td class=\"table-body\">\n\
Easier to port legacy draw code as it looks more like immediate mode drawing. Avoids the complexity of using <span class=\"code\">MRenderItem</span>s.\n\
</td>\n\
<td class=\"table-body\">\n\
Provides a fixed set of shading options, and single texture drawing. The geometry attributes that can be specified are fixed.\n\
</td>\n\
</tr>\n\
<tr class=\"not-ruled-odd-row\">\n\
<td class=\"table-body\">\n\
Supports 2D drawing without having to worry about the matrix transformations required.\n\
</td>\n\
<td class=\"table-body\">\n\
Cannot take advantage of consolidation (since they are not persistent).\n\
</td>\n\
</tr>\n\
<tr class=\"not-ruled-even-row\">\n\
<td class=\"table-body\">\n\
Supports text, icons, lines, face styles, depth priority, basic 2D and 3D primitives, and arbitrary meshes.\n\
</td>\n\
<td class=\"table-body\">\n\
Has no inherent concept of participating based on viewport display modes.\n\
</td>\n\
</tr>\n\
<tr class=\"not-ruled-odd-row\">\n\
<td class=\"table-body\">\n\
Available for all main interfaces including being the only interface for manipulator and tool contexts for 3d viewports and the <span class=\"MenuCascade\">UV Texture Editor</span>.\n\
</td>\n\
<td class=\"table-body\">\n\
Will not participate in post effects and advanced transparency options (as <span class=\"code\">MRenderItem</span>s can).\n\
</td>\n\
</tr>\n\
<tr class=\"not-ruled-even-row\">\n\
<td class=\"table-body\">\n\
One set of code that can be reused across all supported hardware drawing APIs.\n\
</td>\n\
<td class=\"table-body\">\n\
Fixed draw order based on provided properties.\n\
</td>\n\
</tr>\n\
</tbody>\n\
</table>\n\
<p>Each drawable automatically uses the appropriate shader instance and geometry based on the type of draw item. For example, text has an appropriate “text shader”. This knowledge is hidden from the user for simplicity.</p>\n\
<p>The manager checks each drawable and places the drawables into lists which are drawn at different times within the render pipeline. Currently, control over list placement is not exposed in <span class=\'code\'>MUIDrawManager</span>. Users of <span class=\'code\'>MUIDrawManager</span> should be aware that there are explicit properties depending on the list that the drawable is drawn from, and that the interface from which the <span class=\'code\'>MUIDrawManager</span> is used generally determines list placement. For example, queuing drawables from the manager provided via the manipulator interface means that drawables are not depth tested nor written to depth.</p>\n\
<p><span class=\'code\'>MUIDrawManager</span> is not designed for accessing at arbitrary time or at arbitrary places. Access to the interface is thus restricted to a few methods on various API interfaces. Any data and context information management is prepared by the manager and passed to the caller as appropriate. Examples of information that may be passed back to the caller include: DAG path, pass information, and view information. For example, some drawables like heads-up-display elements may need to scale with respect to the viewport rectangle.</p>\n\
<p>The <span class=\'code\'>MUIDrawManager</span> is owned by the renderer and cannot be explicitly constructed.</p>\n\
<p>The following interfaces currently support <span class=\'code\'>MUIDrawManager</span>:</p>\n\
<ul>\n\
<li><span class=\'code\'>MPxDrawOverride</span></li>\n\
<li><span class=\'code\'>MPxGeometryOverride</span></li>\n\
<li><span class=\'code\'>MPxSubSceneOverride</span></li>\n\
<li><span class=\'code\'>MPxManipContainer</span></li>\n\
<li><span class=\'code\'>MPxManipulatorNode</span></li>\n\
<li><span class=\'code\'>MPxContext</span></li>\n\
<li><span class=\'code\'>MUserRenderOperation</span>/<span class=\'code\'>MHUDRender</span>/<span class=\'code\'>MSceneRender</span></li>\n\
</ul>\n\
<p><span class=\'code\'>MPxDrawOverride</span>, <span class=\'code\'>MPxGeometryOverride</span> and <span class=\'code\'>MPxSubSceneOverride</span> have common methods with signatures that can be overridden to use <span class=\'code\'>MUIDrawManager</span>.</p>\n\
<p>The virtual functions are:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual bool hasUIDrawables() const;</span></li>\n\
<li><span class=\'code\'>virtual void addUIDrawables(const MDagPath&amp; objPath, MUIDrawManager&amp; drawManager, const MFrameContext&amp; frameContext, const MUserData* data);</span></li>\n\
</ul>\n\
<p>The method <span class=\'code\'>hasUIDrawables()</span> can be overridden to return true to indicate that the user wishes to add drawables via the <span class=\'code\'>addUIDrawables()</span> method. A false value means to skip calling <span class=\'code\'>addUIDrawables()</span>.</p>\n\
<p>The method <span class=\'code\'>addUIDrawables()</span> is the main method which allows access to <span class=\'code\'>MUIDrawManager</span>. Drawables that are queued here can be thought of as additional render items that can be filtered or bucketed into the appropriate render item list in the pipeline.</p>\n\
<p>Drawables are transient in nature, and as such there is an inherent cost to recreate them each refresh. <span class=\'code\'>MPxSubSceneOverride</span> has an additional virtual method:</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MPxSubSceneOverride::areUIDrawablesDirty()\n\
</pre></div><p>which can be overridden to indicate that drawables are persistent between draw calls.</p>\n\
<p>For manipulators <span class=\'code\'>MPxManipContainer</span> and <span class=\'code\'>MPxManipulatorNode</span>, two override methods provide interaction with <span class=\'code\'>MUIDrawManager</span>:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual void preDrawUI(const M3dView &amp;view);</span></li>\n\
<li><span class=\'code\'>virtual void drawUI(MHWRender::MUIDrawManager&amp; drawManager, const MHWRender::MFrameContext&amp; frameContext) const;</span></li>\n\
</ul>\n\
<p>These methods are only called when the current renderer is Viewport 2.0.</p>\n\
<p><span class=\'code\'>preDrawUI()</span> is always called before <span class=\'code\'>drawUI()</span>. <span class=\'code\'>preDrawUI()</span> allows the user to perform draw data preparation. Then, <span class=\'code\'>drawUI()</span> uses the passed in <span class=\'code\'>MUIDrawManager</span> to draw with the draw data. The drawables queued are rendered after a draw of scene DAG objects, and before the drawing of the HUD (heads-up-display).</p>\n\
<p>For tool contexts, <span class=\'code\'>MPxContext</span> has the following methods available:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual MStatus doPress (MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
<li><span class=\'code\'>virtual MStatus doRelease(MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
<li><span class=\'code\'>virtual MStatus doDrag (MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
<li><span class=\'code\'>virtual MStatus doHold (MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
<li><span class=\'code\'>virtual MStatus doEnterRegion (MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
<li><span class=\'code\'>virtual MStatus doPtrMoved (MEvent &amp; event, MHWRender::MUIDrawManager&amp; drawMgr, const MHWRender::MFrameContext&amp; context);</span></li>\n\
</ul>\n\
<p>These methods are only called when the current renderer is Viewport 2.0.</p>\n\
<p>They can be thought of as the Viewport 2.0 version of <span class=\'code\'>doPress</span>/<span class=\'code\'>doRelease</span>/<span class=\'code\'>doDrag</span>/<span class=\'code\'>doHold</span>/<span class=\'code\'>doEnterRegion</span> and <span class=\'code\'>doPtrMoved</span>. Plug-ins can override these functions to implement the equivalent behavior to that of the default viewport.</p>\n\
<p><span class=\'code\'>MUIDrawManager</span> can also be used for render overrides. For <span class=\'code\'>MUserRenderOperation</span> and <span class=\'code\'>MHUDRender</span>, two methods may be used:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual bool hasUIDrawables() const;</span></li>\n\
<li><span class=\'code\'>virtual void addUIDrawables(MUIDrawManager&amp; drawManager, const MFrameContext&amp; frameContext);</span></li>\n\
</ul>\n\
<p>For <span class=\'code\'>MSceneRender</span>, three functions may be used:</p>\n\
<ul>\n\
<li><span class=\'code\'>virtual bool hasUIDrawables() const;</span></li>\n\
<li><span class=\'code\'>virtual void addPreUIDrawables(MUIDrawManager&amp; drawManager, const MFrameContext&amp; frameContext);</span></li>\n\
<li><span class=\'code\'>virtual void addPostUIDrawables(MUIDrawManager&amp; drawManager, const MFrameContext&amp; frameContext);</span></li>\n\
</ul>\n\
<p>Override these functions to define the drawables to be drawn during the user, HUD, pre-scene, or post-scene render operations.</p>\n\
<p>As mentioned, <span class=\'code\'>MUIDrawManager</span> can be thought of having retained mode drawing. As such, certain states can be set which are applied to the drawables queued after a given state is set.</p>\n\
<p>To control line display attributes, possible line state controls include setting the width and line style.</p>\n\
<p>For example:</p>\n\
<ul>\n\
<li><p><span class=\'code\'>void setLineWidth(float value);</span></p>\n\
<p>sets line width state to a given value in pixels.</p></li>\n\
<li><p><span class=\'code\'>void setLineStyle(LineStyle style);</span></p>\n\
<p>sets line style from a set of pre-defined line styles. These include solid, short dotted, short dashed, dashed, and dotted. For greater control, the method:</p>\n\
<ul>\n\
<li><p><span class=\'code\'>void setLineStyle(unsigned int factor, unsigned short pattern);</span> can be used,</p>\n\
<p>where factor is a multiplier for each bit in the pattern;</p>\n\
<p>where pattern is a bit-mask pattern that determines the fragments of a line to be drawn.</p></li>\n\
</ul></li>\n\
</ul>\n\
<p>For controlling primitive paint style, <span class=\'code\'>setPaintStyle()</span> can be used to allow flat-shaded, stippled, or shaded drawing to be applied. Shaded drawing uses an internal fixed material.</p>\n\
<p>For controlling text, size can be set as a retained mode state (<span class=\'code\'>setFontSize()</span>).</p>\n\
<p><span class=\'code\'>MUIDrawManager</span> supports both 3D and 2D drawable interfaces, and 2D interfaces are generally denoted with &quot;2d&quot; appended to the name of the method. For example l<span class=\'code\'>ine()</span> is for 3D drawable, and <span class=\'code\'>line2d()</span> is for a 2D drawable.</p>\n\
<p>It should be noted that for 2D drawables, only x and y components of a 3d position or vector are used, and the unit for drawing is in screen pixels.</p>\n\
<p>Examples:</p>\n\
<ul>\n\
<li>Manipulator node examples:\n\
<ul>\n\
<li>swissArmyManip, lineManip, footPrintManip, squareScaleManip</li>\n\
</ul></li>\n\
<li><span class=\'code\'>MPxDrawOverride</span> examples:\n\
<ul>\n\
<li>rawfootPrintNode. A Python implementation (<span class=\'code\'>pyFootPrintNode.py</span>) and a C# implementation (<span class=\'code\'>footPrintNode.cs</span>) also exist.</li>\n\
<li>uiDrawManager has a comprehensive list of examples using the interfaces on <span class=\'code\'>MUIDrawManager</span>.</li>\n\
<li>transformDrawNode</li>\n\
<li>simpleEvaluationDraw</li>\n\
</ul></li>\n\
<li><span class=\'code\'>MPxGeometryOverride</span> examples:\n\
<ul>\n\
<li>footPrintNode_GeometryOverride. A Python (<span class=\'code\'>pyFootPrintNode_GeometryOverride.py</span>) version also exists.</li>\n\
<li>geometryReplicator</li>\n\
</ul></li>\n\
<li><span class=\'code\'>MRenderOverride</span> (render override) examples:\n\
<ul>\n\
<li>viewRenderOverride</li>\n\
<li>viewMRenderOverride</li>\n\
</ul></li>\n\
<li><span class=\'code\'>MPxContext</span> (tool context) example:\n\
<ul>\n\
<li>marqueeTool</li>\n\
</ul></li>\n\
<li><span class=\'code\'>MPxSubSceneOverride</span> example:\n\
<ul>\n\
<li>apiMeshSubSceneOverride (apiMeshShape) : Example of using persistent UI drawables.</li>\n\
</ul></li>\n\
</ul>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";