var topic = "<!-- saved from url=(0024)http://docs.autodesk.com -->\n\
<html>\n\
   <head>\n\
<link href=\"../../style/prettify.css\" type=\"text/css\" rel=\"stylesheet\" />\n\
<script type=\"text/javascript\" src=\"../../scripts/prettify.js\"></script>\n\
<script type=\"text/javascript\" src=\"../../scripts/lib/jquery-1.11.1.min.js\"></script><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\"><meta http-equiv=\"Content-Style-Type\" content=\"text/css\"><meta name=\"generator\" content=\"pandoc\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"><script type=\"text/javascript\" src=\"../../scripts/utils/adsk.redirect.js\"></script>\n\
      <title>Deformer Node Example</title>\n\
   </head>\n\
   <body height=\"100%\"><div class=\"body_content\" id=\"body-content\"><style type=\"text/css\">code{white-space: pre;}</style><script>$(document).ready(function () { prettyPrint(); } );</script>\n\
      <div>\n\
         <div class=\"head\">\n\
            <h1>Deformer Node Example</h1>\n\
         </div>\n\
\n\
<div class=\'section\'><a id=\"deformer-node-example\"></a></div>\n\
<p>This section describes the <span class=\'code\'>yTwistNode</span> plug-in, which is an example of a deformer node. This plug-in is available through the Maya Developer Kit.</p>\n\
<div class=\'section\'><a id=\"implementing-the-proxy-deformer-node\"></a><h2 id=\"implementing-the-proxy-deformer-node\">Implementing the Proxy Deformer Node</h2></div>\n\
<p>The <span class=\'code\'>ytwist</span> class inherits from <span class=\'code\'>MPxGeometryFilter</span> and defines a single virtual method, <span class=\'code\'>deform()</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">class yTwist : public MPxGeometryFilter\n\
{\n\
public:\n\
                         yTwist();\n\
    virtual              ~yTwist();\n\
\n\
    static  void*        creator();\n\
    static  MStatus      initialize();\n\
\n\
    // deformation function\n\
    virtual MStatus      deform(MDataBlock&amp;  block,\n\
                         MItGeometry&amp;        iter,\n\
                         const MMatrix&amp;      mat,\n\
                         unsigned int        multiIndex);\n\
\n\
public:\n\
    // yTwist attributes\n\
    static  MObject     angle;          // angle to twist    \n\
    static  MTypeId        id;\n\
\n\
private:\n\
\n\
};\n\
\n\
\n\
</pre></div><div class=\'section\'><a id=\"initializing-the-plug-in\"></a><h2 id=\"initializing-the-plug-in\">Initializing the Plug-in</h2></div>\n\
<p>The new deformer node must be registered with the <span class=\'code\'>registerNode()</span> method of <span class=\'code\'>MFnPlugin</span> when initializing the plug-in.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus initializePlugin( MObject obj )\n\
{\n\
    MStatus result;\n\
    MFnPlugin plugin( obj, PLUGIN_COMPANY, &quot;3.0&quot;, &quot;Any&quot;);\n\
    result = plugin.registerNode( &quot;yTwist&quot;, yTwist::id, yTwist::creator, \n\
                                  yTwist::initialize, MPxNode::kDeformerNode );\n\
    return result;\n\
}\n\
\n\
\n\
</pre></div><p>To remove the deformer node, you must deregister the plug-in with a call to the <span class=\'code\'>deregisterNode()</span> method of <span class=\'code\'>MFnPlugin</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus uninitializePlugin( MObject obj)\n\
{\n\
    MStatus result;\n\
    MFnPlugin plugin( obj );\n\
    result = plugin.deregisterNode( offset::id );\n\
    return result;\n\
}\n\
\n\
</pre></div><div class=\'section\'><a id=\"adding-deformer-attributes\"></a><h2 id=\"adding-deformer-attributes\">Adding Deformer Attributes</h2></div>\n\
<p>The <span class=\'code\'>initialize()</span> method is used to add and configure new attributes to proxy nodes. In the example below, the <span class=\'code\'>angle</span> attribute is added to the node and is made connectable. Any changes to the input attribute <span class=\'code\'>angle</span> affect the output attribute <span class=\'code\'>outputGeom</span>.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus yTwist::initialize()\n\
{\n\
    // local attribute initialization\n\
    //\n\
    MFnNumericAttribute nAttr;\n\
    angle=nAttr.create( &quot;angle&quot;, &quot;fa&quot;, MFnNumericData::kDouble );\n\
        nAttr.setDefault(0.0);\n\
        nAttr.setKeyable(true);\n\
    addAttribute( angle); \n\
\n\
    // affects\n\
    //\n\
    attributeAffects( yTwist::angle, yTwist::outputGeom );\n\
\n\
    return MS::kSuccess;\n\
}\n\
\n\
\n\
</pre></div><div class=\'section\'><a id=\"deform-method\"></a><h2 id=\"deform-method\">Deform Method</h2></div>\n\
<p>The <span class=\'code\'>deform()</span> method implements an algorithm to compute the deformation.</p>\n\
<p>In the <span class=\'code\'>yTwist</span> class, the <span class=\'code\'>deform()</span> method deforms a point with a yTwist algorithm. The geometry data is extracted from the datablock by <span class=\'code\'>MDataHandle</span> and deforms each point of the geometry. The <span class=\'code\'>deform()</span> method returns a <span class=\'code\'>MS::kSuccess</span> to indicate a successful deformation. If not, the deformation encountered problems, such as invalid data input or insufficient memory.</p>\n\
<p>There are four required arguments for this method. The <span class=\'code\'>block</span> argument refers to the node’s datablock where information on the geometry is stored. The <span class=\'code\'>iter</span> argument is the iterator for the geometry to be deformed. The <span class=\'code\'>m</span> is the matrix used to transform points from local space to world space. The <span class=\'code\'>multiIndex</span> is the index of the requested output geometry. In the yTwistNode example, only the <span class=\'code\'>block</span> and the <span class=\'code\'>iter</span> arguments are used.</p>\n\
<div class=\"codeBlock\"><pre class=\"prettyprint\">MStatus\n\
yTwist::deform( MDataBlock&amp; block,\n\
                MItGeometry&amp; iter,\n\
                const MMatrix&amp; /*m*/,\n\
                unsigned int /*multiIndex*/)\n\
\n\
{\n\
    MStatus status = MS::kSuccess;\n\
    \n\
    // determine the angle of the yTwist\n\
    //\n\
    MDataHandle angleData = block.inputValue(angle,&amp;status);\n\
    McheckErr(status, &quot;Error getting angle data handle\\n&quot;);\n\
    double magnitude = angleData.asDouble();\n\
\n\
    // determine the envelope (this is a global scale factor)\n\
    //\n\
    MDataHandle envData = block.inputValue(envelope,&amp;status);\n\
    McheckErr(status, &quot;Error getting envelope data handle\\n&quot;);  \n\
    float env = envData.asFloat();  \n\
\n\
    // iterate through each point in the geometry\n\
    //\n\
    for ( ; !iter.isDone(); iter.next()) {\n\
        \n\
        MPoint pt = iter.position();\n\
\n\
        // do the twist\n\
        //\n\
        double ff = magnitude*pt.y*env;\n\
        if (ff != 0.0) {\n\
            double cct= cos(ff);\n\
            double cst= sin(ff);\n\
            double tt= pt.x*cct-pt.z*cst;\n\
            pt.z= pt.x*cst + pt.z*cct;\n\
            pt.x=tt;;\n\
        }\n\
\n\
        iter.setPosition(pt);\n\
    }\n\
    return status;\n\
}\n\
\n\
</pre></div><p>In this example, no per-vertex weighting is used by the deformer. For an example of per-vertex weighting, see the Developer Kit example offsetNode, which inherits from <span class=\'code\'>MPxDeformerNode</span>, and uses <span class=\'code\'>weightValue()</span> to obtain the weight value for each vertex, CV or lattice point. This example also illustrates how to implement accessory nodes.</p>\n\
<div class=\'section\'><a id=\"other-methods\"></a><h2 id=\"other-methods\">Other methods</h2></div>\n\
<p>There are several other methods in <span class=\'code\'>MPxGeometryFilter</span> that allow the plug-in to:</p>\n\
<ul>\n\
<li>listen to important modification events</li>\n\
<li>handle special case during set editing of points (reconnect using old input/output indices if all points are deleted from the set and then added back in)</li>\n\
</ul>\n\
      <div class=\"footer-block\"><a href=\"../html/ac.cmtdialog.htm\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div></div>\n\
   </div></body>\n\
</html>\n\
";